/* Scrutari - Copyright (c) 2009-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.api.SctContext;
import fr.exemole.sctserver.api.SctContextConstants;
import fr.exemole.sctserver.api.SctServer;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.directory.DirectorySctServerStorage;
import fr.exemole.sctserver.impl.SctServerImpl;
import fr.exemole.sctserver.servlets.selection.SelectionManager;
import fr.exemole.sctserver.tools.SctMessageLocalisationFactory;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TreeMap;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageLocalisationFactory;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariContext {

    public final static String SCRUTARICONTEXT_KEY = "fr.exemole.ScrutariContext";
    private final Map<String, SctContextParameters> nameMap = new HashMap<String, SctContextParameters>();
    private final String scrutariConfFileString;
    private final Timer timer;
    private final MessageLocalisationFactory messageLocalisationFactory;
    private final Map<String, InternalSctContext> sctContextMap = new TreeMap<String, InternalSctContext>();
    private String uuidString = "";
    private String confDirString = "";
    private String varDirString = "";
    private File confDir = null;
    private File varDir = null;
    private Message errorMessage;
    private Lang globalDefaultLangUi = Lang.build("fr");

    ScrutariContext(String scrutariConfFileString) {
        this.scrutariConfFileString = scrutariConfFileString;
        this.messageLocalisationFactory = SctMessageLocalisationFactory.build();
        reload();
        this.timer = new Timer(true);
        timer.scheduleAtFixedRate(new UpdateTask(), getFirstTime(), getPeriod());
    }

    public boolean hasError() {
        return (errorMessage != null);
    }

    public Message getErrorMessage() {
        return errorMessage;
    }

    public SctContext getSctContext(String name) {
        InternalSctContext sctContext = sctContextMap.get(name);
        if (sctContext == null) {
            sctContext = createSctContext(name);
        }
        return sctContext;
    }

    public List<String> getNameList() {
        ArrayList<String> list = new ArrayList<String>();
        list.addAll(nameMap.keySet());
        return list;
    }

    public Lang getGlobalDefaultLangUi() {
        return globalDefaultLangUi;
    }

    public MessageLocalisationFactory getGlobalMessageLocalisationFactory() {
        return messageLocalisationFactory;
    }

    private InternalSctContext createSctContext(String name) {
        SctContextParameters sctContextParameters = nameMap.get(name);
        if (sctContextParameters == null) {
            return null;
        }
        InternalSctContext sctContext;
        synchronized (sctContextParameters) {
            sctContext = sctContextMap.get(name);
            if (sctContext == null) {
                sctContext = buildSctContext(sctContextParameters);
                synchronized (sctContextMap) {
                    sctContextMap.put(name, sctContext);
                }
            }
        }
        return sctContext;
    }

    public static ScrutariContext getScrutariContext(ServletContext servletContext) {
        return (ScrutariContext) servletContext.getAttribute(SCRUTARICONTEXT_KEY);
    }

    public void contextDestroyed() {
        timer.cancel();
    }

    public final synchronized boolean reload() {
        errorMessage = null;
        if (scrutariConfFileString == null) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.INIT_PARAMETER);
            return false;
        }
        File scrutariConfFile = new File(scrutariConfFileString);
        if (!scrutariConfFile.exists()) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.MISSING_CONF_FILE, scrutariConfFileString);
            return false;
        }
        if (scrutariConfFile.isDirectory()) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.CONF_FILE_DIRECTORY, scrutariConfFileString);
            return false;
        }
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(scrutariConfFile);
            return readConf(doc.getDocumentElement());
        } catch (ParserConfigurationException pce) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.PARSER_CONFIG, pce.getMessage());
            return false;
        } catch (IOException ioe) {
            String[] val = new String[2];
            val[0] = scrutariConfFileString;
            val[1] = ioe.getMessage();
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.CONF_FILE_IOERROR, val);
            return false;
        } catch (SAXException sax) {
            String[] val = new String[2];
            val[0] = scrutariConfFileString;
            val[1] = sax.getMessage();
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.XML_PARSE, val);
            return false;
        }
    }

    private Date getFirstTime() {
        Calendar calendar = Calendar.getInstance();
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        calendar.set(Calendar.DAY_OF_MONTH, day + 1);
        calendar.set(Calendar.HOUR_OF_DAY, 1);
        calendar.set(Calendar.MINUTE, 11);
        return calendar.getTime();
    }

    private long getPeriod() {
        return 24 * 60 * 60 * 1000;
    }

    private boolean readConf(Element element_xml) {
        String globalDefaultLangUiString = element_xml.getAttribute("default-lang-ui");
        if (globalDefaultLangUiString.length() > 0) {
            try {
                globalDefaultLangUi = Lang.parse(globalDefaultLangUiString);
            } catch (ParseException pe) {
            }
        }
        String nvUuidString = null;
        String nvConfDirString = null;
        String nvVarDirString = null;
        Map<String, SctContextParameters> hereMap = new HashMap<String, SctContextParameters>();
        NodeList nodeList = element_xml.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node nd = nodeList.item(i);
            if (nd instanceof Element) {
                Element el = (Element) nd;
                String tagName = el.getTagName();
                if ((tagName.equals("engine")) || (tagName.equals("sctserver"))) {
                    String engineName = el.getAttribute("name").trim();
                    if (StringUtils.isTechnicalName(engineName, true)) {
                        SctContextParameters parameters = new SctContextParameters(engineName, globalDefaultLangUi);
                        String defaultLangUiString = el.getAttribute("default-lang-ui");
                        if (defaultLangUiString.length() > 0) {
                            try {
                                parameters.setDefaultLangUi(Lang.parse(defaultLangUiString));
                            } catch (ParseException pe) {
                            }
                        }
                        hereMap.put(engineName, parameters);
                    }
                } else if (tagName.equals("conf-dir")) {
                    nvConfDirString = XMLUtils.getData(el);
                } else if (tagName.equals("var-dir")) {
                    nvVarDirString = XMLUtils.getData(el);
                } else if (tagName.equals("uuid")) {
                    nvUuidString = XMLUtils.getData(el);
                }
            }
        }
        boolean globalChange = false;
        if ((nvUuidString == null) || (nvUuidString.length() == 0)) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.XML_UUID_MISSING);
            return false;
        }
        if ((nvConfDirString == null) || (nvConfDirString.length() == 0)) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.XML_CONFDIR_MISSING);
            return false;
        }
        if ((nvVarDirString == null) || (nvVarDirString.length() == 0)) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.XML_VARDIR_MISSING);
            return false;
        }
        if (!nvUuidString.equals(uuidString)) {
            uuidString = nvUuidString;
            globalChange = true;
        }
        if (!nvConfDirString.equals(confDirString)) {
            File dir = testDir(nvConfDirString, "<conf-dir>");
            if (dir != null) {
                confDirString = nvConfDirString;
                confDir = dir;
                globalChange = true;
            } else {
                return false;
            }
        }
        if (!nvVarDirString.equals(varDirString)) {
            File dir = testDir(nvVarDirString, "<var-dir>");
            if (dir != null) {
                varDirString = nvVarDirString;
                varDir = dir;
                globalChange = true;
            } else {
                return false;
            }
        }
        nameMap.clear();
        nameMap.putAll(hereMap);
        synchronized (sctContextMap) {
            if (globalChange) {
                sctContextMap.clear();
            } else {
                Map<String, InternalSctContext> newMap = new HashMap<String, InternalSctContext>();
                for (SctContextParameters parameters : nameMap.values()) {
                    String engineName = parameters.getEngineName();
                    InternalSctContext sctContext = sctContextMap.get(engineName);
                    if (sctContext != null) {
                        sctContext.checkParameters(parameters);
                        newMap.put(engineName, sctContext);
                    }
                }
                sctContextMap.clear();
                sctContextMap.putAll(newMap);
            }
        }
        return true;
    }

    private File testDir(String path, String elTag) {
        String[] erroval = new String[2];
        erroval[0] = elTag;
        erroval[1] = path;
        File dir = new File(path);
        if (!dir.exists()) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.XML_PATH_NOTEXIST, erroval);
            return null;
        }
        if (!dir.isDirectory()) {
            errorMessage = LocalisationUtils.toMessage(ConfigErrorMessageKeys.XML_PATH_NOTDIR, erroval);
            return null;
        }
        return dir;
    }

    private InternalSctContext buildSctContext(SctContextParameters parameters) {
        String engineName = parameters.getEngineName();
        File sctserverConfDir = new File(confDir, engineName);
        File sctserverVarDir = new File(varDir, engineName);
        if (!sctserverVarDir.exists()) {
            sctserverVarDir.mkdirs();
        }
        File selectionManagerDir = new File(sctserverConfDir, "selection");
        SctServerStorage sctServerStorage = new DirectorySctServerStorage(sctserverConfDir, sctserverVarDir);
        SctServerImpl sctServer = new SctServerImpl(engineName, sctServerStorage, parameters.getDefaultLangUi());
        SelectionManager selectionManager = new SelectionManager(selectionManagerDir);
        InternalSctContext sctContext = new InternalSctContext(sctServer);
        sctContext.putContextObject(SctContextConstants.SELECTIONMANAGER_NAME, selectionManager);
        sctContext.putContextObject(SctContextConstants.UUID_NAME, uuidString);
        return sctContext;
    }


    private class UpdateTask extends TimerTask {

        private UpdateTask() {
        }

        @Override
        public void run() {
            InternalSctContext[] sctContextArray;
            synchronized (sctContextMap) {
                sctContextArray = sctContextMap.values().toArray(new InternalSctContext[sctContextMap.size()]);
            }
            for (InternalSctContext sctContext : sctContextArray) {
                sctContext.sctServer.reload(SctServer.CRON_TYPE);
            }
        }

    }


    private static class SctContextParameters {

        private final String engineName;
        private Lang defaultLangUi;

        private SctContextParameters(String engineName, Lang defaultLangUi) {
            this.engineName = engineName;
            this.defaultLangUi = defaultLangUi;
        }

        private void setDefaultLangUi(Lang defaultLangUi) {
            this.defaultLangUi = defaultLangUi;
        }

        public String getEngineName() {
            return engineName;
        }

        public Lang getDefaultLangUi() {
            return defaultLangUi;
        }

    }


    private static class InternalSctContext implements SctContext {

        private final SctServerImpl sctServer;
        private final Map<String, Object> contextObjectMap = new HashMap<String, Object>();

        private InternalSctContext(SctServerImpl SctServer) {
            this.sctServer = SctServer;
        }

        @Override
        public SctServer getSctServer() {
            return sctServer;
        }

        @Override
        public Object getContextObject(String objectName) {
            return contextObjectMap.get(objectName);
        }

        private void putContextObject(String objectName, Object object) {
            contextObjectMap.put(objectName, object);
        }

        private void checkParameters(SctContextParameters parameters) {
            sctServer.setDefaultLangUi(parameters.getDefaultLangUi());
        }

    }

}
