/* Scrutari - Copyright (c) 2008-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.ConfState;
import fr.exemole.sctserver.api.SctServer;
import fr.exemole.sctserver.api.conf.ConfConstants;
import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.servlets.admin.ConfUpdateCommand;
import fr.exemole.sctserver.servlets.admin.RestartCommand;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.AttributesPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.CategoriesPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.SourcesPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.UriCodesPrinter;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.CommandBoxParametersBuilder;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlConstants;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.PertinencePonderation;


/**
 *
 * @author Vincent Calame
 */
public class IndexPage extends AdminHtmlPage implements ConfConstants {

    private final static HTMLAttributes ERROR_ATTR = HA.classes("scrutari-admin-ErrorMessage");
    private final ConfState confState;

    public IndexPage(SctServer sctServer) {
        init(sctServer);
        confState = sctServer.getConfState();
        CommandBoxParametersBuilder commandBoxParametersBuilder = new CommandBoxParametersBuilder();
        commandBoxParametersBuilder.setCommandKeyDivAttributes(HA.classes("scrutari-admin-CommandKey"));
        commandBoxParametersBuilder.setFormAttributes(HA.action(".").method(HtmlConstants.GET_METHOD));
        commandBoxParametersBuilder.setSubmitDivAttributes(HA.classes("scrutari-admin-Submit"));
        setCommandBoxParameters(commandBoxParametersBuilder.toCommandBoxParameters());
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();
        String dataLog = sctServer.getSctServerStorage().getLogStorage().getLog(SctLogConstants.UPDATE_LOG, SctLogConstants.DATA_LOG);
        if ((dataLog != null) && (!dataLog.isEmpty())) {
            startP();
            startSpan(ERROR_ATTR);
            localize("_ info.admin.updateprocesslog");
            endSpan();
            endP();
            startPre(null);
            escape(dataLog, true);
            endPre();
        }

        printRestartForm();

        printConfState();

        end();
    }

    private void printConfState() {
        startH2(null);
        localize("_ title.sctconf.files");
        endH2();
        short state = confState.getGlobalState();
        boolean printConfDetails = false;
        switch (state) {
            case ConfState.GLOBAL_CONFIO_ERROR:
                startP();
                startSpan(ERROR_ATTR);
                localize("_ error.global.conf_io");
                append(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_UNDEFINED:
                startP();
                startSpan(ERROR_ATTR);
                localize("_ error.global.conf_null");
                append(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_OK:
                startUl();
                printConfFileLi(SOURCES_NAME);
                printConfFileLi(ATTRIBUTES_NAME);
                printConfFileLi(CATEGORIES_NAME);
                printConfFileLi(URI_CODES_NAME);
                printConfFileLi(FEED_NAME);
                printConfFileLi(OPTIONS_NAME);
                endUl();
                printConfDetails = true;
                break;
        }
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction(".");
        commandBox.setCommandFamily("SCT");
        commandBox.setCommandString(ConfUpdateCommand.COMMANDNAME);
        commandBox.setCommandKey(ConfUpdateCommand.COMMANDKEY);
        startCommandBox(commandBox);
        endCommandBox("_ submit.admin.confupdate");
        if (printConfDetails) {
            startH2(null);
            localize("_ title.sctconf.details");
            endH2();
            printSourcesDetails();
            printAttributesDetails();
            printCategoriesDetails();
            printUriCodesDetails();
            printFeedDetails();
            printOptionsDetails();
        }
    }

    private void printConfFileLi(String name) {
        startLi();
        printFileInfo(name, true);
        endLi();
    }

    private void printSourcesDetails() {
        startH3(HA.id(SOURCES_NAME));
        localize("_ title.sctconf.sources");
        append(" / ");
        printFileInfo(SOURCES_NAME, false);
        endH3();
        printConfLog(SOURCES_NAME);
        SourcesPrinter.printScrutariSourceList(this);
    }

    private void printAttributesDetails() {
        startH3(HA.id(ATTRIBUTES_NAME));
        localize("_ title.sctconf.attributes");
        append(" / ");
        printFileInfo(ATTRIBUTES_NAME, false);
        endH3();
        printConfLog(ATTRIBUTES_NAME);
        AttributeKeyCoder attributeKeyCoder = sctServer.getScrutariSession().getScrutariDB().getAttributeKeyCoder();
        if (attributeKeyCoder.getAttributeDefCount() == 0) {
            startP();
            startSpan(HA.classes("scrutari-admin-Info"));
            localize("_ info.sctconf.none_attributes");
            endSpan();
            endP();
            return;
        }
        printAttributesList(attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.PRIMARY_GROUP), "_ title.sctconf.attributes_primary");
        printAttributesList(attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.SECONDARY_GROUP), "_ title.sctconf.attributes_secondary");
        printAttributesList(attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.TECHNICAL_GROUP), "_ title.sctconf.attributes_technical");
    }

    private void printAttributesList(AttributeDefList attributeDefList, String titleMessageKey) {
        if (attributeDefList.getAttributeDefCount() == 0) {
            return;
        }
        startH4(null);
        localize(titleMessageKey);
        endH4();
        AttributesPrinter.printAttributesList(this, attributeDefList);
    }

    private void printCategoriesDetails() {
        startH3(HA.id(CATEGORIES_NAME));
        localize("_ title.sctconf.categories");
        append(" / ");
        printFileInfo(CATEGORIES_NAME, false);
        endH3();
        printConfLog(CATEGORIES_NAME);
        CategoriesPrinter.printCategories(this);
    }

    private void printUriCodesDetails() {
        startH3(HA.id(URI_CODES_NAME));
        localize("_ title.sctconf.uricodes");
        append(" / ");
        printFileInfo(URI_CODES_NAME, false);
        endH3();
        printConfLog(URI_CODES_NAME);
        UriCodesPrinter.printUriCodes(this);
    }

    private void printFeedDetails() {
        startH3(HA.id(FEED_NAME));
        localize("_ title.sctconf.feed");
        append(" / ");
        printFileInfo(FEED_NAME, false);
        endH3();
        printConfLog(FEED_NAME);
    }

    private void printOptionsDetails() {
        startH3(HA.id(OPTIONS_NAME));
        localize("_ title.sctconf.options");
        append(" / ");
        printFileInfo(OPTIONS_NAME, false);
        endH3();
        printConfLog(OPTIONS_NAME);
        startH4(null);
        localize("_ title.sctconf.options_ponderation");
        endH4();
        GlobalSearchOptions globalSearchOptions = sctServer.getScrutariSession().getGlobalSearchOptions();
        PertinencePonderation pertinencePonderation = globalSearchOptions.getPertinencePonderation();
        startTable();
        printPonderatioRow("_ label.sctconf.ponderation_occurrence", pertinencePonderation.getOccurrencePonderation());
        printPonderatioRow("_ label.sctconf.ponderation_date", pertinencePonderation.getDatePonderation());
        printPonderatioRow("_ label.sctconf.ponderation_origin", pertinencePonderation.getOriginPonderation());
        endTable();
    }

    private void printPonderatioRow(String key, float f) {
        startRow();
        startCell();
        HtmlPrinterUtils.printLabelWithColon(this, key);
        endCell();
        startCell();
        append(String.valueOf(Math.round(f * 100)));
        append("%");
        endCell();
        endRow();
    }

    private void printRestartForm() throws IOException {
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction(".");
        commandBox.setCommandFamily("SCT");
        commandBox.setCommandString(RestartCommand.COMMANDNAME);
        commandBox.setCommandKey(RestartCommand.COMMANDKEY);
        startCommandBox(commandBox);
        startP();
        startLabel(null);
        appendInput(HA.type(HtmlConstants.CHECKBOX_TYPE).name(RestartCommand.FORCE_PARAMNAME).value("1"));
        localize("_ label.admin.forcerestart");
        endLabel();
        endP();
        endCommandBox("_ submit.admin.restart");
    }

    private void printConfLog(String name) {
        short fileState = confState.getFileState(name);
        switch (fileState) {
            case FILE_STATE_UNKNOWN:
            case FILE_MISSING:
            case FILE_OK:
                return;
        }
        String logContent = sctServer.getSctServerStorage().getLogStorage().getLog(SctLogConstants.CONF_LOG, name);
        if (logContent == null) {
            return;
        }
        startP();
        localize("_ title.admin.log");
        endP();
        startPre(HA.classes("scrutari-admin-Log"));
        escape(logContent, true);
        endPre();
    }

    private void printFileInfo(String name, boolean withLink) {
        short fileState = confState.getFileState(name);
        if (withLink) {
            startA(HA.href("#" + name));
        }
        append(name);
        append(".xml");
        if (withLink) {
            endA();
        }
        append(" (");
        startSpan(HA.classes(getSpanClass(fileState)));
        localize(getFileStateMessageKey(fileState));
        endSpan();
        append(")");
    }

    private static String getSpanClass(short fileState) {
        switch (fileState) {
            case FILE_STATE_UNKNOWN:
                return "scrutari-state-Neutral";
            case FILE_MISSING:
                return "scrutari-state-Neutral";
            case FILE_XML_MALFORMED:
                return "scrutari-state-Error";
            case FILE_XML_ERROR:
                return "scrutari-state-Error";
            case FILE_XML_WARNING:
                return "scrutari-state-Warning";
            case FILE_OK:
                return "scrutari-state-Ok";
            default:
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
    }

    private static String getFileStateMessageKey(short fileState) {
        switch (fileState) {
            case FILE_STATE_UNKNOWN:
                return "_ state.file.unknownstate";
            case FILE_MISSING:
                return "_ state.file.missing";
            case FILE_XML_MALFORMED:
                return "_ state.file.xml_malformed";
            case FILE_XML_ERROR:
                return "_ state.file.xml_error";
            case FILE_XML_WARNING:
                return "_ state.file.xml_warning";
            case FILE_OK:
                return "_ state.file.ok";
            default:
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
    }

}
