/* ScrutariLib_DB - Copyright (c) 2008-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.collation.map.CollatedKeyMap;
import net.mapeadores.util.text.collation.map.SortedCollatedKeyMap;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.db.api.LexieAccessProvider;
import net.scrutari.db.api.caches.CacheIterator;
import net.scrutari.db.api.caches.LexieCacheReader;
import net.scrutari.db.api.caches.LexieCacheReaderFactory;
import net.scrutari.lexie.FicheLexification;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.lexie.MotcleLexification;
import net.scrutari.lexie.ScrutariLexieUnit;
import net.scrutari.lexie.ScrutariLexieUnitList;


/**
 *
 * @author Vincent Calame
 */
public class LexieAccessProviderBuilder {

    private LexieAccessProviderBuilder() {

    }

    public static LexieAccessProvider build(LexieCacheReaderFactory lexieCacheReaderFactory) {
        Map<Lang, CollatedKeyMap<Long>> lexieIdMapByLangMap = new HashMap<Lang, CollatedKeyMap<Long>>();
        int lexieCount = 0;
        CacheIterator<LexieCacheReaderFactory.LexieUnitInfo> iterator = lexieCacheReaderFactory.newLexieUnitInfoIterator();
        while (iterator.hasNext()) {
            LexieCacheReaderFactory.LexieUnitInfo info = iterator.next();
            Lang langInteger = info.getLang();
            CollatedKeyMap<Long> lexieIdMap = lexieIdMapByLangMap.get(langInteger);
            if (lexieIdMap == null) {
                lexieIdMap = new SortedCollatedKeyMap<Long>(langInteger.toLocale());
                lexieIdMapByLangMap.put(langInteger, lexieIdMap);
            }
            ScrutariLexieUnitList list = info.getScrutariLexieUnitList();
            int count = list.getScrutariLexieUnitCount();
            for (int i = 0; i < count; i++) {
                ScrutariLexieUnit unit = list.getScrutariLexieUnit(i);
                lexieIdMap.putValueByCollatedKey(unit.getCollatedLexie(), unit.getLexieId());
                lexieCount++;
            }
        }
        iterator.close();
        InternalLexieAccessProvider internalLexieAccessProvider = new InternalLexieAccessProvider(lexieCacheReaderFactory, lexieIdMapByLangMap, lexieCount);
        return internalLexieAccessProvider;
    }


    private static class InternalLexieAccessProvider implements LexieAccessProvider {

        private final LexieCacheReaderFactory lexieCacheReaderFactory;
        private final Map<Lang, CollatedKeyMap<Long>> lexieIdMapByLangMap;
        private final int lexieCount;

        private InternalLexieAccessProvider(LexieCacheReaderFactory lexieCacheReaderFactory, Map<Lang, CollatedKeyMap<Long>> lexieIdMapByLangMap, int lexieCount) {
            this.lexieCacheReaderFactory = lexieCacheReaderFactory;
            this.lexieIdMapByLangMap = lexieIdMapByLangMap;
            this.lexieCount = lexieCount;
        }

        @Override
        public LexieAccess openLexieAccess() {
            return new InternalLexieAccess();
        }


        private class InternalLexieAccess implements LexieAccess {

            private LexieCacheReader lexieCacheReader;

            private InternalLexieAccess() {
            }

            @Override
            public int getLexieCount() {
                return lexieCount;
            }

            @Override
            public CollatedKeyMap<Long> getLexieIdMapByLang(Lang lang) {
                return lexieIdMapByLangMap.get(lang);
            }

            @Override
            public LexieOccurrencesList getLexieOccurrencesList(long lexieId, int subsetCode) {
                testCacheReader();
                return lexieCacheReader.getLexieOccurrencesList(lexieId, subsetCode);
            }

            @Override
            public ScrutariLexieUnitList getScrutariLexieUnitList(Lang lang) {
                testCacheReader();
                return lexieCacheReader.getScrutariLexieUnitList(lang);
            }

            @Override
            public FicheLexification getFicheLexification(int codefiche) {
                testCacheReader();
                return lexieCacheReader.getFicheLexification(codefiche);
            }

            @Override
            public MotcleLexification getMotcleLexification(int codemotcle) {
                testCacheReader();
                return lexieCacheReader.getMotcleLexification(codemotcle);
            }

            @Override
            public void close() {
                if (lexieCacheReader != null) {
                    lexieCacheReader.close();
                }
            }

            private void testCacheReader() {
                if (lexieCacheReader == null) {
                    lexieCacheReader = lexieCacheReaderFactory.newReader();
                }
            }

        }

    }

}
