/* ScrutariLib_DB - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.codes;

import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.db.api.CodeMessageKeys;
import net.scrutari.db.api.DataAccess;


/**
 *
 * @author Vincent Calame
 */
public class CodeParser {

    private CodeParser() {
    }

    public static int toCode(DataAccess dataAccess, String s, ProcessMessageHandler processMessageHandler, String uriBase, short uriType) {
        ScrutariDataURI scrutariDataURI;
        char carac1 = s.charAt(0);
        if ((carac1 >= '0') && (carac1 <= '9')) {
            int idx = s.indexOf('/');
            if (idx == -1) {
                try {
                    int code = Integer.parseInt(s);
                    if (!testCode(dataAccess, code, uriType)) {
                        processMessageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, CodeMessageKeys.UNKNOWN_CODE, s);
                        return -1;
                    }
                    return code;
                } catch (NumberFormatException nfe) {
                    processMessageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, CodeMessageKeys.MALFORMED_NOT_INTEGER, s);
                    return -1;
                }
            } else {
                String s1 = s.substring(0, idx);
                try {
                    int code1 = Integer.parseInt(s1);
                    ScrutariDataURI startScrutariDataURI = getParentScrutariDataURI(dataAccess, code1);
                    if (startScrutariDataURI == null) {
                        processMessageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, CodeMessageKeys.UNKNOWN_URI, s1 + " (" + s + ")");
                        return -1;
                    }
                    String s2 = s.substring(idx);
                    String uriString = ScrutariDataURI.typeToSchemeString(uriType) + ':' + startScrutariDataURI.getPath() + s2;
                    try {
                        scrutariDataURI = URIParser.parse(uriString);
                    } catch (URIParseException uriParseException) {
                        processMessageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, CodeMessageKeys.MALFORMED_URI, s2 + " (" + s + ")");
                        return -1;
                    }
                } catch (NumberFormatException nfe) {
                    processMessageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, CodeMessageKeys.MALFORMED_NOT_INTEGER, s1 + " (" + s + ")");
                    return -1;
                }
            }
        } else {
            boolean isRelative = false;
            int idx = s.indexOf(':');
            String uriString = s;
            if (idx == -1) {
                isRelative = true;
                uriString = uriBase + uriString;
            }
            try {
                scrutariDataURI = URIParser.parse(uriString);

            } catch (URIParseException uriParseException) {
                processMessageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, CodeMessageKeys.MALFORMED_URI, s);
                return -1;
            }
            if (scrutariDataURI.getType() != uriType) {
                if (isRelative) {
                    throw new IllegalArgumentException("uriBase and uriType are not of the same type");
                } else {
                    processMessageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, CodeMessageKeys.WRONG_URI_TYPE, s);
                    return -1;
                }
            }
        }
        int code = dataAccess.getCode(scrutariDataURI);
        if (code == -1) {
            processMessageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, CodeMessageKeys.UNKNOWN_URI, s);
        }
        return code;
    }

    private static boolean testCode(DataAccess dataAccess, int code, short uriType) {
        switch (uriType) {
            case ScrutariDataURI.BASEURI_TYPE:
                return (dataAccess.getBaseDataByCode(code) != null);
            case ScrutariDataURI.CORPUSURI_TYPE:
                return (dataAccess.getCorpusDataByCode(code) != null);
            case ScrutariDataURI.THESAURUSURI_TYPE:
                return (dataAccess.getThesaurusDataByCode(code) != null);
            case ScrutariDataURI.FICHEURI_TYPE:
                return (dataAccess.getFicheDataByCode(code) != null);
            case ScrutariDataURI.MOTCLEURI_TYPE:
                return (dataAccess.getMotcleDataByCode(code) != null);
            default:
                throw new SwitchException("uriType = " + uriType);
        }

    }

    private static ScrutariDataURI getParentScrutariDataURI(DataAccess dataAccess, int code) {
        BaseData baseData = dataAccess.getBaseDataByCode(code);
        if (baseData != null) {
            return baseData.getBaseURI();
        }
        CorpusData corpusData = dataAccess.getCorpusDataByCode(code);
        if (corpusData != null) {
            return corpusData.getCorpusURI();
        }
        ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(code);
        if (thesaurusData != null) {
            return thesaurusData.getThesaurusURI();
        }
        return null;
    }

}
