/* ScrutariLib_DB - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.stats;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.localisation.LangListUtils;
import net.mapeadores.util.primitives.PrimUtils;
import net.mapeadores.util.text.Libelle;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.IndexationCodeList;
import net.scrutari.db.api.stats.CountStats;
import net.scrutari.db.api.stats.FicheStats;
import net.scrutari.db.api.stats.ScrutariDBStats;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariDBStatsBuilder {

    private final CountStatsBuilder engineCountStatsBuilder = CountStatsBuilder.newEngineInstance();
    private final Map<Integer, CountStats> countStatsMap = new HashMap<Integer, CountStats>();
    private final DataAccess dataAccess;
    private final LexieAccess lexieAccess;
    private final LangueBundle allMotcleLang = new LangueBundle();
    private final Map<Integer, FicheStatsBuilder> ficheStatsBuilderMap = new HashMap<Integer, FicheStatsBuilder>();
    private final FicheStatsBuilder engineFicheStatsBuilder = new FicheStatsBuilder();
    private final IndexationInfoMap infoMap;

    private ScrutariDBStatsBuilder(DataAccess dataAccess, LexieAccess lexieAccess) {
        this.dataAccess = dataAccess;
        this.lexieAccess = lexieAccess;
        infoMap = new IndexationInfoMap(dataAccess);
    }

    private void check() {
        engineCountStatsBuilder.increaseLexieCount(lexieAccess.getLexieCount());
        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            checkBase(baseData);
        }
    }

    private void checkBase(BaseData baseData) {
        engineCountStatsBuilder.increaseBaseCount(1);
        CountStatsBuilder baseCountStatsBuilder = CountStatsBuilder.newBaseInstance();
        FicheStatsBuilder baseFicheStatsBuilder = new FicheStatsBuilder();
        int corpusCount = baseData.getCorpusCount();
        for (int j = 0; j < corpusCount; j++) {
            CorpusData corpusData = dataAccess.getCorpusDataByCode(baseData.getCodeCorpus(j));
            checkCorpus(corpusData, baseCountStatsBuilder, baseFicheStatsBuilder);
        }
        ficheStatsBuilderMap.put(baseData.getCodeBase(), baseFicheStatsBuilder);
        int thesaurusCount = baseData.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            int codethesaurus = baseData.getCodeThesaurus(i);
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(codethesaurus);
            checkThesaurus(thesaurusData, baseCountStatsBuilder);
        }
        countStatsMap.put(baseData.getCodeBase(), baseCountStatsBuilder.toCountStats());
    }

    private void checkCorpus(CorpusData corpusData, CountStatsBuilder baseCountStatsBuilder, FicheStatsBuilder baseFicheStatsBuilder) {
        baseCountStatsBuilder.increaseCorpusCount(1);
        engineCountStatsBuilder.increaseCorpusCount(1);
        CountStatsBuilder corpusCountStatsBuilder = CountStatsBuilder.newCorpusInstance();
        FicheStatsBuilder corpusFicheStatsBuilder = new FicheStatsBuilder();
        int ficheCount = corpusData.getFicheCount();
        corpusCountStatsBuilder.increaseFicheCount(ficheCount);
        baseCountStatsBuilder.increaseFicheCount(ficheCount);
        engineCountStatsBuilder.increaseFicheCount(ficheCount);
        for (int j = 0; j < ficheCount; j++) {
            Lang lang = dataAccess.getFicheLang(corpusData.getCodeFiche(j));
            corpusFicheStatsBuilder.addOne(lang);
            baseFicheStatsBuilder.addOne(lang);
            engineFicheStatsBuilder.addOne(lang);
        }
        int codecorpus = corpusData.getCodeCorpus();
        ficheStatsBuilderMap.put(codecorpus, corpusFicheStatsBuilder);
        IndexationInfo indexationInfo = infoMap.getIndexationInfo(codecorpus);
        int indexationCount = indexationInfo.getIndexationCount();
        corpusCountStatsBuilder.increaseIndexationCount(indexationCount);
        baseCountStatsBuilder.increaseIndexationCount(indexationCount);
        engineCountStatsBuilder.increaseIndexationCount(indexationCount);
        corpusCountStatsBuilder.increaseMotcleCount(indexationInfo.getIndexedCount());
        countStatsMap.put(codecorpus, corpusCountStatsBuilder.toCountStats());
    }

    private void checkThesaurus(ThesaurusData thesaurusData, CountStatsBuilder baseCountStatsBuilder) {
        baseCountStatsBuilder.increaseThesaurusCount(1);
        engineCountStatsBuilder.increaseThesaurusCount(1);
        CountStatsBuilder thesaurusCountStatsBuilder = CountStatsBuilder.newThesaurusInstance();
        int motcleCount = thesaurusData.getMotcleCount();
        thesaurusCountStatsBuilder.increaseMotcleCount(motcleCount);
        baseCountStatsBuilder.increaseMotcleCount(motcleCount);
        engineCountStatsBuilder.increaseMotcleCount(motcleCount);
        for (int i = 0; i < motcleCount; i++) {
            int codemotcle = thesaurusData.getCodeMotcle(i);
            MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
            int libelleCount = motcleData.getLibelleCount();
            for (int k = 0; k < libelleCount; k++) {
                Libelle libelle = motcleData.getLibelleByIndex(k);
                allMotcleLang.put(libelle.getLang());
            }
            IndexationInfo infoMotcle = infoMap.getIndexationInfo(codemotcle);
            CountStatsBuilder motcleCountStatsBuilder = CountStatsBuilder.newMotcleInstance();
            motcleCountStatsBuilder.increaseFicheCount(infoMotcle.getIndexationCount());
            countStatsMap.put(codemotcle, motcleCountStatsBuilder.toCountStats());
        }
        int codethesaurus = thesaurusData.getCodeThesaurus();
        IndexationInfo info = infoMap.getIndexationInfo(codethesaurus);
        thesaurusCountStatsBuilder.increaseFicheCount(info.getIndexedCount());
        thesaurusCountStatsBuilder.increaseIndexationCount(info.getIndexationCount());
        countStatsMap.put(codethesaurus, thesaurusCountStatsBuilder.toCountStats());
    }

    private ScrutariDBStats toScrutariDBStats() {
        LangList allMotcleLangList = allMotcleLang.toLangList();
        Map<Integer, FicheStats> ficheStatsMap = new HashMap<Integer, FicheStats>();
        for (Map.Entry<Integer, FicheStatsBuilder> entry : ficheStatsBuilderMap.entrySet()) {
            ficheStatsMap.put(entry.getKey(), entry.getValue().toFicheStats());
        }
        return new InternalScrutariDBStats(engineCountStatsBuilder.toCountStats(), countStatsMap, engineFicheStatsBuilder.toFicheStats(), allMotcleLangList, ficheStatsMap, infoMap.indexationSet);
    }

    public static ScrutariDBStats build(DataAccess dataAccess, LexieAccess lexieAccess) {
        ScrutariDBStatsBuilder builder = new ScrutariDBStatsBuilder(dataAccess, lexieAccess);
        builder.check();
        return builder.toScrutariDBStats();

    }


    private static class InternalScrutariDBStats implements ScrutariDBStats {

        private final CountStats engineCountStats;
        private final Map<Integer, CountStats> countStatsMap;
        private final FicheStats engineFicheStats;
        private final Map<Integer, FicheStats> ficheStatsMap;
        private final LangList allMotcleLangList;
        private final Set<Long> indexationSet;

        private InternalScrutariDBStats(CountStats engineCountStats, Map<Integer, CountStats> countStatsMap, FicheStats engineFicheStats, LangList allMotcleLangList, Map<Integer, FicheStats> ficheStatsMap, Set<Long> indexationSet) {
            this.engineCountStats = engineCountStats;
            this.countStatsMap = countStatsMap;
            this.allMotcleLangList = allMotcleLangList;
            this.engineFicheStats = engineFicheStats;
            this.ficheStatsMap = ficheStatsMap;
            this.indexationSet = indexationSet;
        }

        @Override
        public CountStats getEngineCountStats() {
            return engineCountStats;
        }

        @Override
        public CountStats getCountStats(int code) {
            CountStats countStats = countStatsMap.get(code);
            return countStats;
        }

        @Override
        public LangList getAllMotcleLangList() {
            return allMotcleLangList;
        }

        @Override
        public FicheStats getEngineFicheStats() {
            return engineFicheStats;
        }


        @Override
        public FicheStats getFicheStats(int code) {
            FicheStats ficheStats = ficheStatsMap.get(code);
            if (ficheStats == null) {
                return FicheStatsBuilder.EMPTY_FICHESTATS;
            } else {
                return ficheStats;
            }
        }

        @Override
        public boolean hasIndexation(int codecorpus, int codethesaurus) {
            long lg = PrimUtils.toLong(codecorpus, codethesaurus);
            return indexationSet.contains(lg);
        }

    }

    private static class IndexationInfo {

        private int indexationCount = 0;
        private Set<Integer> codeSet;

        private IndexationInfo() {
        }

        public int getIndexationCount() {
            return indexationCount;
        }

        public int getIndexedCount() {
            if (codeSet == null) {
                return 0;
            }
            return codeSet.size();
        }

        private void incrementCount() {
            indexationCount++;
        }

        private void addIndexedCode(int code) {
            if (codeSet == null) {
                codeSet = new HashSet<Integer>();
            }
            codeSet.add(code);
        }

    }


    private static class IndexationInfoMap {

        private final Map<Integer, IndexationInfo> map = new HashMap<Integer, IndexationInfo>();
        private final Set<Long> indexationSet = new HashSet<Long>();

        private IndexationInfoMap(DataAccess dataAccess) {
            int thesaurusCount = dataAccess.getThesaurusCount();
            CodeList corpusCodeList = dataAccess.getCorpusCodeList();
            int corpusCount = corpusCodeList.getCodeCount();
            for (int i = 0; i < thesaurusCount; i++) {
                ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
                int codethesaurus = thesaurusData.getCodeThesaurus();
                int motcleCount = thesaurusData.getMotcleCount();
                for (int j = 0; j < motcleCount; j++) {
                    int codemotcle = thesaurusData.getCodeMotcle(j);
                    for (int k = 0; k < corpusCount; k++) {
                        int codecorpus = corpusCodeList.getCode(k);
                        IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpus);
                        int ficheCount = indexationCodeList.getCodeCount();
                        if (ficheCount > 0) {
                            indexationSet.add(PrimUtils.toLong(codecorpus, codethesaurus));
                            for (int l = 0; l < ficheCount; l++) {
                                int codefiche = indexationCodeList.getCode(l);
                                IndexationInfo corpusInfo = getOrCreate(codecorpus);
                                IndexationInfo thesaurusInfo = getOrCreate(codethesaurus);
                                IndexationInfo motcleInfo = getOrCreate(codemotcle);
                                IndexationInfo ficheInfo = getOrCreate(codefiche);
                                motcleInfo.incrementCount();
                                ficheInfo.incrementCount();
                                corpusInfo.incrementCount();
                                thesaurusInfo.incrementCount();
                                corpusInfo.addIndexedCode(codemotcle);
                                thesaurusInfo.addIndexedCode(codefiche);
                            }
                        }
                    }
                }
            }
        }

        public IndexationInfo getIndexationInfo(int code) {
            IndexationInfo indexationInfo = map.get(code);
            if (indexationInfo == null) {
                return new IndexationInfo();
            }
            return indexationInfo;
        }

        private IndexationInfo getOrCreate(int code) {
            IndexationInfo indexationInfo = map.get(code);
            if (indexationInfo == null) {
                indexationInfo = new IndexationInfo();
                map.put(code, indexationInfo);
            }
            return indexationInfo;
        }

    }


    private static class LangueBundle {

        private final Set<Lang> langSet = new LinkedHashSet<Lang>();

        private LangueBundle() {
        }

        private void add(LangueBundle langBundle) {
            langSet.addAll(langBundle.langSet);
        }

        private void put(Lang lang) {
            langSet.add(lang);
        }

        private LangList toLangList() {
            return LangListUtils.fromCollection(langSet);
        }

    }

}
