/* ScrutariLib_Data - Copyright (c) 2005-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.datauri.CorpusURI;


/**
 *
 * @author Vincent Calame
 */
public final class CorpusData implements Serializable {

    public final static int INTITULECORPUS_FIELDNUMBER = -11;
    public final static int INTITULEFICHE_FIELDNUMBER = -22;
    private final CorpusURI corpusURI;
    private final int codecorpus;
    private final int codebase;
    private final int complementCount;
    private final CorpusLangData[] corpusLangDataArray;
    private final int[] codesFiches;

    private CorpusData(CorpusURI corpusURI, int codecorpus, int codebase, int complementCount, CorpusLangData[] corpusLangDataArray, int[] codesFiches) {
        this.corpusURI = corpusURI;
        this.codecorpus = codecorpus;
        this.codebase = codebase;
        this.complementCount = complementCount;
        this.corpusLangDataArray = corpusLangDataArray;
        this.codesFiches = codesFiches;
    }

    public CorpusURI getCorpusURI() {
        return corpusURI;
    }

    /**
     * Retourne le nombre maximum de champs complémentaires.
     */
    public int getComplementMaxNumber() {
        return complementCount;
    }

    /**
     * Retourne le code du corpus.
     */
    public int getCodeCorpus() {
        return codecorpus;
    }

    /**
     * Retourne le code de la base.
     */
    public int getCodeBase() {
        return codebase;
    }

    /**
     * Retourne le total de fiches du corpus.
     */
    public int getFicheCount() {
        return codesFiches.length;
    }

    /**
     * Retourne le code de la i-ième fiche du corpus.
     */
    public int getCodeFiche(int i) {
        return codesFiches[i];
    }

    public CorpusLangData getLangDataByLang(Lang lang) {
        int length = corpusLangDataArray.length;
        for (int i = 0; i < length; i++) {
            CorpusLangData corpusLangData = corpusLangDataArray[i];
            if (corpusLangData.lang.equals(lang)) {
                return corpusLangData;
            }
        }
        return null;
    }

    public int getLangDataCount() {
        return corpusLangDataArray.length;
    }

    public CorpusLangData getLangDataByIndex(int index) {
        return corpusLangDataArray[index];
    }

    public static CorpusData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        CorpusURI corpusURI = (CorpusURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        int codecorpus = primitivesReader.readInt();
        int codebase = primitivesReader.readInt();
        int complementCount = primitivesReader.readInt();
        int langDataLength = primitivesReader.readInt();
        CorpusLangData[] corpusLangDataArray = new CorpusLangData[langDataLength];
        for (int i = 0; i < langDataLength; i++) {
            Lang lang = Lang.build(primitivesReader.readString());
            CorpusLangData corpusLangData = new CorpusLangData(lang, complementCount);
            readLibelle(corpusLangData, primitivesReader);
            readLibelle(corpusLangData, primitivesReader);
            for (int j = 0; j < complementCount; j++) {
                readLibelle(corpusLangData, primitivesReader);
            }
            corpusLangDataArray[i] = corpusLangData;
        }
        int fichesLength = primitivesReader.readInt();
        int[] codesFiches = new int[fichesLength];
        for (int i = 0; i < fichesLength; i++) {
            codesFiches[i] = primitivesReader.readInt();
        }
        return new CorpusData(corpusURI, codecorpus, codebase, complementCount, corpusLangDataArray, codesFiches);
    }

    public static void toPrimitives(CorpusData corpusData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(corpusData.corpusURI, primitivesWriter);
        primitivesWriter.writeInt(corpusData.codecorpus);
        primitivesWriter.writeInt(corpusData.codebase);
        int complementCount = corpusData.complementCount;
        primitivesWriter.writeInt(complementCount);
        int langDataLength = corpusData.corpusLangDataArray.length;
        primitivesWriter.writeInt(langDataLength);
        for (int i = 0; i < langDataLength; i++) {
            CorpusLangData corpusLangData = corpusData.corpusLangDataArray[i];
            primitivesWriter.writeString(corpusLangData.lang.toString());
            writeLibelle(INTITULECORPUS_FIELDNUMBER, corpusLangData, primitivesWriter);
            writeLibelle(INTITULEFICHE_FIELDNUMBER, corpusLangData, primitivesWriter);
            for (int j = 0; j < complementCount; j++) {
                writeLibelle(j + 1, corpusLangData, primitivesWriter);
            }
        }
        int fichesLength = corpusData.codesFiches.length;
        primitivesWriter.writeInt(fichesLength);
        for (int i = 0; i < fichesLength; i++) {
            primitivesWriter.writeInt(corpusData.codesFiches[i]);
        }
    }

    private static void writeLibelle(int fieldNumber, CorpusLangData corpusLangData, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeInt(fieldNumber);
        String libelleString = corpusLangData.getLibelleString(fieldNumber);
        primitivesWriter.writeString(libelleString);
    }

    private static void readLibelle(CorpusLangData corpusLangData, PrimitivesReader primitivesReader) throws IOException {
        int fieldNumber = primitivesReader.readInt();
        String libelleString = primitivesReader.readString();
        if (libelleString != null) {
            corpusLangData.setLibelle(DataLibelleBuilder.toLibelle(corpusLangData.lang, libelleString), fieldNumber);
        }
    }


    public static class CorpusLangData {

        private final Lang lang;
        private Libelle corpusLibelle;
        private Libelle ficheLibelle;
        private final Libelle[] complementsLibelle;

        private CorpusLangData(Lang lang, int complementCount) {
            this.lang = lang;
            complementsLibelle = new Libelle[complementCount];
        }

        public Libelle getLibelle(int fieldNumber) {
            if (fieldNumber == INTITULECORPUS_FIELDNUMBER) {
                return corpusLibelle;
            }
            if (fieldNumber == INTITULEFICHE_FIELDNUMBER) {
                return ficheLibelle;
            }
            return complementsLibelle[fieldNumber - 1];
        }

        public String getLibelleString(int fieldNumber) {
            Libelle libelle = getLibelle(fieldNumber);
            if (libelle == null) {
                return null;
            }
            return libelle.getLibelleString();
        }

        public Lang getLang() {
            return lang;
        }

        private void setLibelle(Libelle libelle, int fieldNumber) {
            if (fieldNumber == INTITULECORPUS_FIELDNUMBER) {
                corpusLibelle = libelle;
            } else if (fieldNumber == INTITULEFICHE_FIELDNUMBER) {
                ficheLibelle = libelle;
            } else {
                complementsLibelle[fieldNumber - 1] = libelle;
            }
        }

    }


    public static class Builder {

        private final CorpusURI corpusURI;
        private final Map<Lang, CorpusLangDataBuilder> corpusLangDataMap = new LinkedHashMap<Lang, CorpusLangDataBuilder>();
        private final List<Integer> codeficheList = new ArrayList<Integer>();
        private String hrefParent;
        private int complementCount = 0;

        public Builder(CorpusURI corpusURI) {
            this.corpusURI = corpusURI;
        }

        public CorpusURI getCorpusURI() {
            return corpusURI;
        }

        public void addLibelle(int fieldNumber, Libelle libelle) {
            if (libelle == null) {
                return;
            }
            if (fieldNumber > complementCount) {
                throw new IllegalArgumentException("fieldNumber > libelleListOfList.size()");
            }
            CorpusLangDataBuilder builder = getCorpusLangDataBuilder(libelle.getLang());
            builder.putLibelle(fieldNumber, libelle);
        }

        public int createNewComplement() {
            complementCount++;
            return complementCount;
        }

        public int getComplementMaxNumber() {
            return complementCount;
        }

        public void addCodeFiche(int codefiche) {
            codeficheList.add(codefiche);
        }

        public CorpusData toCorpusData(int codecorpus, int codebase) {
            int ficheCount = codeficheList.size();
            int[] codesFiches = new int[ficheCount];
            for (int i = 0; i < ficheCount; i++) {
                codesFiches[i] = codeficheList.get(i);
            }
            int langCount = corpusLangDataMap.size();
            CorpusLangData[] corpusLangDataArray = new CorpusLangData[langCount];
            int p = 0;
            for (CorpusLangDataBuilder builder : corpusLangDataMap.values()) {
                corpusLangDataArray[p] = builder.toCorpusLangData(complementCount);
                p++;
            }
            return new CorpusData(corpusURI, codecorpus, codebase, complementCount, corpusLangDataArray, codesFiches);
        }

        public String getHrefParent() {
            return hrefParent;
        }

        public void setHrefParent(String hrefParent) {
            this.hrefParent = StringUtils.nullifie(hrefParent);
        }

        private CorpusLangDataBuilder getCorpusLangDataBuilder(Lang lang) {
            CorpusLangDataBuilder builder = corpusLangDataMap.get(lang);
            if (builder == null) {
                builder = new CorpusLangDataBuilder(lang);
                corpusLangDataMap.put(lang, builder);
            }
            return builder;
        }

    }


    private static class CorpusLangDataBuilder {

        private final Lang lang;
        private final Map<Integer, Libelle> libelleMap = new HashMap<Integer, Libelle>();

        private CorpusLangDataBuilder(Lang lang) {
            this.lang = lang;
        }

        private void putLibelle(int fieldNumber, Libelle libelle) {
            libelleMap.put(fieldNumber, libelle);
        }

        private CorpusLangData toCorpusLangData(int complementCount) {
            CorpusLangData corpusLangData = new CorpusLangData(lang, complementCount);
            for (Map.Entry<Integer, Libelle> entry : libelleMap.entrySet()) {
                corpusLangData.setLibelle(entry.getValue(), entry.getKey());
            }
            return corpusLangData;
        }

    }

}
