/* ScrutariLib_SearchEngine - Copyright (c) 2013-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.filters;

import java.util.HashSet;
import java.util.Set;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.logicaloperation.SimpleOperand;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.db.api.URITreeProvider;
import net.scrutari.searchengine.api.operands.OperandMessageKeys;
import net.scrutari.searchengine.api.operands.OperandsConstants;
import net.scrutari.searchengine.api.operands.filters.CodeOperand;
import net.scrutari.searchengine.tools.OperandCheck;


/**
 *
 * @author Vincent Calame
 */
final class AddCheck {

    private AddCheck() {
    }

    static OperandCheck check(SimpleOperand simpleOperand, OperandConverterParameters operandConverterParameters) {
        boolean acceptMode = simpleOperand.isAcceptMode();
        String operandBody = simpleOperand.getBody();
        AddParameters addParameters = parse(simpleOperand, operandConverterParameters);
        if (addParameters == null) {
            return OperandCheck.UNDEFINED_OPERANDECHECK;
        }
        URITreeProvider uriTreeProvider = operandConverterParameters.getUriTreeProvider();
        DataAccess dataAccess = operandConverterParameters.getDataAccess();
        Set<Integer> codeSet = new HashSet<Integer>();
        Set<Integer> corpusSet = new HashSet<Integer>();
        int uriTreeCount = uriTreeProvider.getAddURITreeDateCount();
        int ficheCount = 0;
        for (int i = 0; i < uriTreeCount; i++) {
            ScrutariDBName scrutariDBName = uriTreeProvider.getScrutariDBName(i);
            int dateInteger = DateInteger.fromDate(scrutariDBName.getDate());
            if (!addParameters.test(dateInteger, ficheCount)) {
                break;
            }
            URITree uriTree = uriTreeProvider.getAddURITree(i);
            ficheCount = ficheCount + check(codeSet, corpusSet, uriTree, dataAccess);
        }
        CodeOperand operand = new AddOperand(acceptMode, codeSet, operandBody);
        return new OperandCheck(corpusSet, operand);
    }

    private static int check(Set<Integer> codeSet, Set<Integer> corpusSet, URITree uriTree, DataAccess dataAccess) {
        int baseCount = uriTree.getBaseCount();
        int result = 0;
        for (int i = 0; i < baseCount; i++) {
            BaseNode baseNode = uriTree.getBaseNode(i);
            BaseURI baseURI = baseNode.getBaseURI();
            BaseData baseData = (BaseData) dataAccess.getScrutariData(baseURI);
            if (baseData == null) {
                continue;
            }
            result = result + checkBase(baseNode, codeSet, corpusSet, dataAccess);
        }
        return result;
    }

    private static int checkBase(BaseNode baseNode, Set<Integer> codeSet, Set<Integer> corpusSet, DataAccess dataAccess) {
        int corpusCount = baseNode.getCorpusCount();
        int result = 0;
        for (int j = 0; j < corpusCount; j++) {
            CorpusNode corpusNode = baseNode.getCorpusNode(j);
            CorpusURI corpusURI = corpusNode.getCorpusURI();
            CorpusData corpusData = (CorpusData) dataAccess.getScrutariData(corpusURI);
            if (corpusData == null) {
                continue;
            }
            int ficheCount = checkCorpus(corpusNode, codeSet, dataAccess);
            if (ficheCount > 0) {
                result = result + ficheCount;
                corpusSet.add(corpusData.getCodeCorpus());
            }
        }
        return result;
    }

    private static int checkCorpus(CorpusNode corpusNode, Set<Integer> codeSet, DataAccess dataAccess) {
        int ficheCount = corpusNode.getFicheCount();
        int result = 0;
        for (int j = 0; j < ficheCount; j++) {
            FicheNode ficheNode = corpusNode.getFicheNode(j);
            FicheURI ficheURI = ficheNode.getFicheURI();
            int codefiche = dataAccess.getCode(ficheURI);
            if (codefiche != -1) {
                result++;
                codeSet.add(codefiche);
            }
        }
        return result;
    }

    private static AddParameters parse(SimpleOperand simpleOperand, OperandConverterParameters operandConverterParameters) {
        String operandBody = simpleOperand.getBody();
        ProcessMessageHandler processMessageHandler = operandConverterParameters.getProcessMessageHandler();
        String[] tokens = StringUtils.getTechnicalTokens(operandBody, false);
        int length = tokens.length;
        int dayLimit = 0;
        int minimumFicheCount = 0;
        boolean malformed = false;
        if (length == 0) {
            malformed = true;
        }
        for (int i = 0; i < length; i++) {
            String token = tokens[i];
            String[] subTokens;
            if (token.indexOf('=') != -1) {
                subTokens = StringUtils.getTokens(token, '=', StringUtils.EMPTY_EXCLUDE); //Version antérieure à la révision 1731
            } else {
                subTokens = StringUtils.getTokens(token, '-', StringUtils.EMPTY_EXCLUDE);
            }
            int length2 = subTokens.length;
            if (length2 != 2) {
                malformed = true;
                break;
            }
            int value;
            try {
                value = Integer.parseInt(subTokens[1]);
            } catch (NumberFormatException nfe) {
                malformed = true;
                break;
            }
            if (value < 1) {
                malformed = true;
                break;
            }
            String key = subTokens[0];
            if (key.equals("d")) {
                dayLimit = value;
            } else if (key.equals("f")) {
                minimumFicheCount = value;
            }
        }
        if ((dayLimit == 0) && (minimumFicheCount == 0)) {
            malformed = true;
        }
        if (malformed) {
            processMessageHandler.addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.WRONG_ADD_OPERAND, simpleOperand.getOperandString());
            return null;
        }
        int floorDate = 0;
        if (dayLimit != 0) {
            int currentDate = DateInteger.current();
            floorDate = DateInteger.roll(currentDate, -dayLimit);
        }

        return new AddParameters(floorDate, minimumFicheCount);
    }


    private static class AddParameters {

        private int floorDate = 0;
        private int minimumFicheCount = 0;

        private AddParameters(int floorDate, int minimumFicheCount) {
            this.floorDate = floorDate;
            this.minimumFicheCount = minimumFicheCount;
        }

        private boolean test(int dateInteger, int ficheCount) {
            if ((floorDate != 0) && (dateInteger > floorDate)) {
                return true;
            }
            if ((minimumFicheCount != 0) && (ficheCount < minimumFicheCount)) {
                return true;
            }
            return false;
        }

    }


    private static class AddOperand implements CodeOperand {

        private final boolean acceptMode;
        private final Set<Integer> codeSet;
        private final String bodyString;

        private AddOperand(boolean acceptMode, Set<Integer> codeSet, String bodyString) {
            this.acceptMode = acceptMode;
            this.codeSet = codeSet;
            this.bodyString = bodyString;
        }

        @Override
        public boolean acceptCode(int code) {
            boolean test = codeSet.contains(code);
            if (acceptMode) {
                return test;
            } else {
                return !test;
            }
        }

        @Override
        public String toString() {
            StringBuilder buf = new StringBuilder();
            if (!acceptMode) {
                buf.append('!');
            }
            buf.append(OperandsConstants.ADD_TYPE);
            buf.append(':');
            buf.append(bodyString);
            return buf.toString();
        }

    }

}
