/* ScrutariLib_SearchEngine - Copyright (c) 2013-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.filters;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.LogicalOperationUtils;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.SimpleOperand;
import net.mapeadores.util.logicaloperation.SubOperand;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.searchengine.api.operands.OperandMessageKeys;
import net.scrutari.searchengine.api.operands.OperandsConstants;
import net.scrutari.searchengine.api.operands.filters.UnknownOperandTypeException;
import net.scrutari.searchengine.tools.OperandCheck;


/**
 *
 * @author Vincent Calame
 */
public class FilterOperandConverterEngine {

    /**
     * Retourne OperandCheck.UNDEFINED_OPERANDECHECK en cas d'impossibilité de
     * définition sans que cela soit une erreur (par exemple, si l'opérande se
     * limite à la déclaration du type). Ne retourne jamais nul. Envoie une
     * exception si defaultType est incorrect (defaultType peut être nul).
     */
    public static OperandCheck convert(Operand operand, CodeList corpusCodeList, String defaultType, OperandConverterParameters operandConverterParameters) throws UnknownOperandTypeException {
        Object defaultTypeObject = null;
        if (defaultType != null) {
            defaultTypeObject = checkType(defaultType);
            if (defaultTypeObject == null) {
                try {
                    defaultTypeObject = AttributeKey.parse(defaultType);
                } catch (ParseException pe) {
                    throw new UnknownOperandTypeException(defaultType);
                }
            }
        }
        if (corpusCodeList == null) {
            corpusCodeList = operandConverterParameters.getDataAccess().getCorpusCodeList();
        }
        if (operand instanceof SubOperand) {
            return convertSubOperand((SubOperand) operand, corpusCodeList, defaultTypeObject, operandConverterParameters);
        } else if (operand instanceof SimpleOperand) {
            return convertSimpleOperand((SimpleOperand) operand, corpusCodeList, defaultTypeObject, operandConverterParameters);
        } else {
            throw new IllegalArgumentException("Unkonwn Operand instance = " + operand.getClass().getName());
        }
    }

    private static OperandCheck convertSubOperand(SubOperand subOperand, CodeList corpusCodeList, Object defaultTypeObject, OperandConverterParameters operandConverterParameters) {
        SubOperandCheckBuilder builder = new SubOperandCheckBuilder(subOperand);
        int count = subOperand.getOperandCount();
        boolean noneBreak = false;
        for (int i = 0; i < count; i++) {
            Operand operand = subOperand.getOperand(i);
            OperandCheck operandCheck;
            if (operand instanceof SimpleOperand) {
                operandCheck = convertSimpleOperand((SimpleOperand) operand, corpusCodeList, defaultTypeObject, operandConverterParameters);
            } else if (operand instanceof SubOperand) {
                operandCheck = convertSubOperand((SubOperand) operand, corpusCodeList, defaultTypeObject, operandConverterParameters);
            } else {
                throw new IllegalArgumentException("Unkonwn Operand instance = " + operand.getClass().getName());
            }
            if (operandCheck.equals(OperandCheck.NONE_OPERANDECHECK)) {
                if (subOperand.getOperator() == LogicalOperationConstants.INTERSECTION_OPERATOR) {
                    noneBreak = true;
                    break;
                }
            } else if (!operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
                builder.addOperandCheck(operandCheck);
            }
        }
        if (noneBreak) {
            return OperandCheck.NONE_OPERANDECHECK;
        }
        if (builder.isEmpty()) {
            return OperandCheck.UNDEFINED_OPERANDECHECK;
        }
        return builder.toOperandCheck();
    }

    /*
     * Retourne null en cas d'erreur et OperandCheck.UNDEFINED_OPERANDECHECK en cas
     * d'impossibilité de définition sans que cela soit une erreur (par exemple, si l'opérande
     * se limite à la déclaration du type
     *
     */
    private static OperandCheck convertSimpleOperand(SimpleOperand simpleOperand, CodeList corpusCodeList, Object defaultTypeObject, OperandConverterParameters operandConverterParameters) {
        String scope = simpleOperand.getScope();
        Object typeObject;
        if (scope.isEmpty()) {
            if (defaultTypeObject == null) {
                operandConverterParameters.getProcessMessageHandler().addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.UNKNOWN_OPERAND_TYPE, simpleOperand.getOperandString());
                return OperandCheck.UNDEFINED_OPERANDECHECK;
            } else {
                typeObject = defaultTypeObject;
            }
        } else {
            typeObject = checkType(scope);
            if (typeObject == null) {
                try {
                    typeObject = AttributeKey.parse(scope);
                } catch (ParseException pe) {
                    operandConverterParameters.getProcessMessageHandler().addMessage(OperandMessageKeys.ERROR_CATEGORY, OperandMessageKeys.UNKNOWN_OPERAND_TYPE, simpleOperand.getOperandString());
                    return OperandCheck.UNDEFINED_OPERANDECHECK;
                }
            }
        }
        if (simpleOperand.getBody().length() == 0) {
            return OperandCheck.UNDEFINED_OPERANDECHECK;
        }
        if (typeObject instanceof AttributeKey) {
            return AttributeCheck.check((AttributeKey) typeObject, simpleOperand, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.MOTCLE_TYPE)) {
            return MotcleCheck.check(simpleOperand, corpusCodeList, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.THESAURUS_TYPE)) {
            return ThesaurusCheck.check(simpleOperand, corpusCodeList, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.DATE_TYPE)) {
            return DateCheck.check(simpleOperand, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.QID_TYPE)) {
            return PreviousResultCheck.check(simpleOperand, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.ADD_TYPE)) {
            return AddCheck.check(simpleOperand, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.BBOX_TYPE)) {
            return BboxCheck.check(simpleOperand, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.CIRCLE_TYPE)) {
            return CircleCheck.check(simpleOperand, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.CORPUS_TYPE)) {
            return CorpusCheck.check(simpleOperand, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.BASE_TYPE)) {
            return BaseCheck.check(simpleOperand, operandConverterParameters);
        } else if (typeObject.equals(OperandsConstants.LANG_TYPE)) {
            return LangCheck.check(simpleOperand, operandConverterParameters);
        } else {
            throw new SwitchException("Unknown type = " + typeObject.toString());
        }
    }

    private static String checkType(String s) {
        if (s.equals(OperandsConstants.MOTCLE_TYPE)) {
            return OperandsConstants.MOTCLE_TYPE;
        } else if (s.equals(OperandsConstants.THESAURUS_TYPE)) {
            return OperandsConstants.THESAURUS_TYPE;
        } else if (s.equals(OperandsConstants.DATE_TYPE)) {
            return OperandsConstants.DATE_TYPE;
        } else if (s.equals(OperandsConstants.QID_TYPE)) {
            return OperandsConstants.QID_TYPE;
        } else if (s.equals(OperandsConstants.ADD_TYPE)) {
            return OperandsConstants.ADD_TYPE;
        } else if (s.equals(OperandsConstants.BBOX_TYPE)) {
            return OperandsConstants.BBOX_TYPE;
        } else if (s.equals(OperandsConstants.CIRCLE_TYPE)) {
            return OperandsConstants.CIRCLE_TYPE;
        } else if (s.equals(OperandsConstants.CORPUS_TYPE)) {
            return OperandsConstants.CORPUS_TYPE;
        } else if (s.equals(OperandsConstants.BASE_TYPE)) {
            return OperandsConstants.BASE_TYPE;
        } else if (s.equals(OperandsConstants.LANG_TYPE)) {
            return OperandsConstants.LANG_TYPE;
        } else {
            return null;
        }
    }


    private static class SubOperandCheckBuilder {

        private final List<OperandCheck> operandCheckList = new ArrayList<OperandCheck>();
        private final short operator;

        private SubOperandCheckBuilder(SubOperand subOperand) {
            this.operator = subOperand.getOperator();
        }

        private void addOperandCheck(OperandCheck operandCheck) {
            operandCheckList.add(operandCheck);
        }

        private boolean isEmpty() {
            return operandCheckList.isEmpty();
        }

        private OperandCheck toOperandCheck() {
            boolean intersection = (operator == LogicalOperationConstants.INTERSECTION_OPERATOR);
            int size = operandCheckList.size();
            Operand[] array = new Operand[size];
            Set<Integer> resultCodeSet = null;
            for (int i = 0; i < size; i++) {
                OperandCheck operandCheck = operandCheckList.get(i);
                Set<Integer> codeSet = operandCheck.getCodeSet();
                if (i == 0) {
                    if (codeSet != null) {
                        resultCodeSet = new HashSet<Integer>(codeSet);
                    }
                } else {
                    if (intersection) {
                        if (codeSet != null) {
                            if (resultCodeSet != null) {
                                resultCodeSet.retainAll(codeSet);
                            } else {
                                resultCodeSet = new HashSet<Integer>(codeSet);
                            }
                        }
                    } else {
                        if (codeSet == null) {
                            resultCodeSet = null;
                        } else if (resultCodeSet != null) {
                            resultCodeSet.addAll(codeSet);
                        }
                    }
                }
                array[i] = operandCheck.getOperand();
            }
            SubOperand subOperand = LogicalOperationUtils.toSubOperand(array, operator);
            return new OperandCheck(resultCodeSet, subOperand);
        }

    }

}
