/* ScrutariLib_Xml - Copyright (c) 2008-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml;

import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.LibelleElementHandler;
import net.mapeadores.util.xml.xmlpull.SimpleElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.caches.DataCacheWriter;
import static net.scrutari.xml.ErrorKeys.SEVERE_XML_BASEROOT;
import net.scrutari.xml.xmlpull.CorpusHandler;
import net.scrutari.xml.xmlpull.ThesaurusHandler;
import net.scrutari.xml.xmlpull.XmlPullUtils;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;


/**
 *
 * @author Vincent Calame
 */
public class BaseXmlEngine implements ErrorKeys {

    private final XmlPullParserFactory factory;
    private final DataCacheWriter dataCacheWriter;
    private final DataCoder dataCoder;
    private final ScrutariXmlErrorHandler scrutariXmlErrorHandler;
    private final Parameters parameters;
    private BaseData.Builder baseDataBuilder;
    public static final int AUTHORITY_UUID = 1;
    public static final int BASE_NAME = 2;
    public static final int BASE_ICON = 3;
    public static final int LANG = 4;

    public BaseXmlEngine(DataCacheWriter dataCacheWriter, DataCoder dataCoder, ScrutariXmlErrorHandler scrutariXmlErrorHandler) {
        this.dataCacheWriter = dataCacheWriter;
        this.dataCoder = dataCoder;
        this.scrutariXmlErrorHandler = scrutariXmlErrorHandler;
        parameters = new Parameters(dataCacheWriter, dataCoder, scrutariXmlErrorHandler);
        try {
            this.factory = XmlPullParserFactory.newInstance(System.getProperty(XmlPullParserFactory.PROPERTY_NAME), null);
        } catch (XmlPullParserException xppe) {
            throw new NestedLibraryException(xppe);
        }
    }

    public BaseURI run(String scrutariSourceName, InputStream inputStream) throws XmlPullParserException, IOException {
        XmlPullParser xpp = factory.newPullParser();
        xpp.setInput(inputStream, null);
        baseDataBuilder = null;
        RootHandler rootHandler = null;
        try {
            int eventType = xpp.getEventType();
            do {
                if (eventType == XmlPullParser.START_TAG) {
                    String tagName = xpp.getName();
                    if (!IndexationXmlEngine.isElement(tagName)) {
                        if (rootHandler == null) {
                            if (tagName.equals("base")) {
                                rootHandler = new RootHandler();
                            } else {
                                scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_BASEROOT);
                                break;
                            }
                        } else {
                            Map<String, String> map = null;
                            int count = xpp.getAttributeCount();
                            if (count > 0) {
                                map = new HashMap<String, String>();
                                for (int i = 0; i < count; i++) {
                                    map.put(xpp.getAttributeName(i), xpp.getAttributeValue(i));
                                }
                            }
                            rootHandler.processStartElement(xpp.getName(), map);
                        }
                    }
                } else if (eventType == XmlPullParser.END_TAG) {
                    String tagName = xpp.getName();
                    if (!IndexationXmlEngine.isElement(tagName)) {
                        rootHandler.processEndElement(tagName);
                    }
                } else if (eventType == XmlPullParser.TEXT) {
                    if (rootHandler != null) {
                        rootHandler.processText(xpp.getText());
                    }
                }
                eventType = xpp.next();
            } while (eventType != XmlPullParser.END_DOCUMENT);
            if (rootHandler != null) {
                if (baseDataBuilder == null) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_BASE_MISSINGMETADATA);
                    return null;
                } else {
                    BaseData baseData = baseDataBuilder.toBaseData(parameters.getCodeBase(), scrutariSourceName);
                    dataCacheWriter.cacheBaseData(baseData);
                    return baseDataBuilder.getBaseURI();
                }
            } else {
                return null;
            }
        } catch (InterruptionException ie) {
            return null;
        }
    }


    private class RootHandler extends DefaultElementHandler {

        private RootHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (baseDataBuilder == null) {
                if (!tagname.equals("base-metadata")) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_BASE_MISSINGMETADATA);
                    throw new InterruptionException();
                }
                return new MetadataHandler();
            } else if (tagname.equals("corpus")) {
                try {
                    return new CorpusHandler(attributeMap, parameters);
                } catch (IllegalArgumentException iae) {
                    return null;
                }

            } else if (tagname.equals("thesaurus")) {
                try {
                    return new ThesaurusHandler(attributeMap, parameters);
                } catch (IllegalArgumentException iae) {
                    return null;
                }
            } else {
                return null;
            }
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof CorpusHandler) {
                CorpusData corpusData = ((CorpusHandler) elementHandler).flushCorpusData();
                if (corpusData != null) {
                    baseDataBuilder.addCodeCorpus(corpusData.getCodeCorpus());
                }
            } else if (elementHandler instanceof ThesaurusHandler) {
                ThesaurusData thesaurusData = ((ThesaurusHandler) elementHandler).flushThesaurusData();
                if (thesaurusData != null) {
                    baseDataBuilder.addCodeThesaurus(thesaurusData.getCodeThesaurus());
                }
            }
        }

        BaseData.Builder getBaseDataBuilder() {
            return baseDataBuilder;
        }

    }


    private class MetadataHandler extends DefaultElementHandler {

        private String authorityuuid = null;
        private String baseName = null;

        private MetadataHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("authority-uuid")) {
                return new SimpleElementHandler(AUTHORITY_UUID);
            }
            if (tagname.equals("base-name")) {
                return new SimpleElementHandler(BASE_NAME);
            }
            initBaseDataBuilder();
            if (tagname.equals("base-icon")) {
                return new SimpleElementHandler(BASE_ICON);
            } else if (tagname.equals("intitule-short")) {
                return new IntituleHandler(BaseData.INTITULE_SHORT);
            } else if (tagname.equals("intitule-long")) {
                return new IntituleHandler(BaseData.INTITULE_LONG);
            } else if (tagname.equals("langs-ui")) {
                return new LangUiHandler();
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            if (text.length() == 0) {
                text = null;
            }
            switch (type) {
                case AUTHORITY_UUID:
                    authorityuuid = text;
                    break;
                case BASE_NAME:
                    baseName = text;
                    break;
                case BASE_ICON:
                    baseDataBuilder.setBaseIcon(text);
                    break;
            }
        }

        private void initBaseDataBuilder() {
            if (baseDataBuilder != null) {
                return;
            }
            if ((authorityuuid == null) || (baseName == null)) {
                if (authorityuuid == null) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_AUTHORITYUUID);
                }
                if (baseName == null) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_NOMBASE);
                }
                throw new InterruptionException();
            }
            BaseURI baseURI = new BaseURI(authorityuuid, baseName);
            if (dataCoder.checkCode(baseURI) != -1) {
                scrutariXmlErrorHandler.addSevereLog(SEVERE_DATABASE_DOUBLE, baseURI);
                throw new InterruptionException();
            }
            baseDataBuilder = new BaseData.Builder(baseURI);
            parameters.newBase(baseURI, dataCoder.getCode(baseURI));
        }

    }


    private class IntituleHandler extends DefaultElementHandler {

        private final short type;

        private IntituleHandler(short type) {
            this.type = type;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch (ParseException lie) {
                    scrutariXmlErrorHandler.addWarningLog(WARNING_XML_BASE_LANGERROR, baseDataBuilder.getBaseURI());
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Libelle libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                baseDataBuilder.addLibelle(type, libelle);
            }

        }

    }


    private class LangUiHandler extends DefaultElementHandler {

        private LangUiHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("lang")) {
                return new SimpleElementHandler(LANG);
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            try {
                Lang lang = Lang.parse(text);
                baseDataBuilder.addLangUI(lang);
            } catch (ParseException ile) {
                scrutariXmlErrorHandler.addWarningLog(WARNING_XML_BASE_LANGERROR, baseDataBuilder.getBaseURI());
            }
        }

    }


    private static class InterruptionException extends RuntimeException {
    }

}
