/* ScrutariLib_Xml - Copyright (c) 2008-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml.xmlpull;

import java.text.ParseException;
import java.util.Map;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.LibelleElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.xml.ErrorKeys;
import net.scrutari.xml.Parameters;
import net.scrutari.xml.ScrutariXmlErrorHandler;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusHandler extends DefaultElementHandler implements ErrorKeys {

    private final DataCacheWriter dataCacheWriter;
    private final DataCoder dataCoder;
    private final ScrutariXmlErrorHandler scrutariXmlErrorHandler;
    private final ThesaurusURI thesaurusURI;
    private final int codebase;
    private final int codethesaurus;
    private final ThesaurusData.Builder thesaurusDataBuilder;
    private final MotcleHandler motcleHandler = new MotcleHandler();
    private boolean metadataDone = false;

    public ThesaurusHandler(Map<String, String> attributeMap, Parameters parameters) {
        this.scrutariXmlErrorHandler = parameters.getScrutariXmlErrorHandler();
        String thesaurusName = getAttributeString(attributeMap, "thesaurus-name");
        if (thesaurusName == null) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_THESAURUS_MISSINGTHESAURUSNAME);
            throw new IllegalArgumentException();
        }
        this.dataCacheWriter = parameters.getDataCacheWriter();
        this.dataCoder = parameters.getDataCoder();
        this.thesaurusURI = new ThesaurusURI(parameters.getBaseURI(), thesaurusName);
        if (dataCoder.checkCode(thesaurusURI) != -1) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_THESAURUS_DOUBLETHESAURUSNAME, thesaurusURI);
            throw new IllegalArgumentException();
        }
        this.codethesaurus = dataCoder.getCode(thesaurusURI);
        this.codebase = parameters.getCodeBase();
        thesaurusDataBuilder = new ThesaurusData.Builder(thesaurusURI);
    }

    public ThesaurusData flushThesaurusData() {
        if (!metadataDone) {
            return null;
        }
        ThesaurusData thesaurusData = thesaurusDataBuilder.toThesaurusData(codethesaurus, codebase);
        dataCacheWriter.cacheThesaurusData(thesaurusData);
        return thesaurusData;
    }

    @Override
    public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
        if (!metadataDone) {
            if (tagname.equals("thesaurus-metadata")) {
                metadataDone = true;
                return new MetadataHandler();
            } else {
                scrutariXmlErrorHandler.addWarningLog(WARNING_XML_THESAURUS_MISSINGMETADATA, thesaurusURI);
                return null;
            }
        } else {
            if (tagname.equals("motcle")) {
                String motcleId = getAttributeString(attributeMap, "motcle-id");
                if (motcleId == null) {
                    scrutariXmlErrorHandler.addWarningLog(WARNING_XML_MOTCLE_MISSINGMOTCLEID);
                    return null;
                }
                MotcleURI motcleURI = new MotcleURI(thesaurusURI, motcleId);
                if (dataCoder.checkCode(motcleURI) != -1) {
                    scrutariXmlErrorHandler.addWarningLog(WARNING_XML_MOTCLE_DOUBLEMOTCLEID, motcleURI);
                    return null;
                }
                int codemotcle = dataCoder.getCode(motcleURI);
                motcleHandler.newMotcle(motcleURI, codemotcle);
                return motcleHandler;
            } else {
                return null;
            }
        }
    }

    @Override
    public void endSubElement(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof MotcleHandler) {
            MotcleHandler mh = (MotcleHandler) elementHandler;
            MotcleData.Builder motcleDataBuilder = mh.getMotcleDataBuilder();
            int codemotcle = mh.getCodeMotcle();
            MotcleData motcleData = motcleDataBuilder.toMotcleData(codemotcle, codethesaurus, codebase);
            dataCacheWriter.cacheMotcleData(motcleData);
            thesaurusDataBuilder.addCodeMotcle(codemotcle);
        }
    }


    private class MetadataHandler extends DefaultElementHandler {

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("intitule-thesaurus")) {
                return new IntituleHandler();
            } else {
                return null;
            }
        }

    }


    private class IntituleHandler extends DefaultElementHandler {

        private IntituleHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch (ParseException lie) {
                    scrutariXmlErrorHandler.addWarningLog(WARNING_XML_THESAURUS_LANGERROR, thesaurusURI);
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Libelle libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                thesaurusDataBuilder.addLibelle(libelle);
            }
        }

    }


    private class MotcleHandler extends DefaultElementHandler {

        private MotcleURI motcleURI;
        private int codemotcle;
        private MotcleData.Builder motcleDataBuilder;

        private MotcleHandler() {
        }

        private void newMotcle(MotcleURI motcleURI, int codemotcle) {
            this.motcleURI = motcleURI;
            this.codemotcle = codemotcle;
            motcleDataBuilder = new MotcleData.Builder(motcleURI);
        }

        private MotcleData.Builder getMotcleDataBuilder() {
            return motcleDataBuilder;
        }

        private int getCodeMotcle() {
            return codemotcle;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch (ParseException lie) {
                    scrutariXmlErrorHandler.addWarningLog(WARNING_XML_MOTCLE_LANGERROR, motcleURI);
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Libelle libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                motcleDataBuilder.addLibelle(libelle);
            }
        }

    }

}
