/* SctServer - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.impl;

import fr.exemole.sctserver.api.ScrutariSourceManager;
import fr.exemole.sctserver.api.collect.CollectErrorHandler;
import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.collect.ScrutariSourceList;
import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.tools.collect.CollectEngine;
import fr.exemole.sctserver.tools.collect.CollectUtils;
import fr.exemole.sctserver.tools.collect.ScrutariSourceStorage;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.primitives.DateInteger;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariSourceManagerImpl implements ScrutariSourceManager, ScrutariSourceList {

    private final Map<String, InternalScrutariSource> scrutariSourceMap = new HashMap<String, InternalScrutariSource>();
    private final List<InternalScrutariSource> scrutariSourceList = new ArrayList<InternalScrutariSource>();
    private SctServerStorage sctServerStorage;

    public ScrutariSourceManagerImpl(SctServerStorage sctServerStorage) {
        this.sctServerStorage = sctServerStorage;
    }

    @Override
    public ScrutariSource getScrutariSouce(String scrutariSourceName) {
        return scrutariSourceMap.get(scrutariSourceName);
    }

    @Override
    public int getScrutariSourceCount() {
        return scrutariSourceList.size();
    }

    @Override
    public ScrutariSource getScrutariSource(int i) {
        return scrutariSourceList.get(i);
    }

    @Override
    public ScrutariSourceList getScrutariSourceList() {
        return this;
    }

    @Override
    public void reloadConf(ConfDirectory confDirectory) {
        ScrutariSourceDef[] scrutariSourceDefArray = confDirectory.getScrutariSourceDefArray();
        if (scrutariSourceDefArray == null) {
            scrutariSourceList.clear();
            scrutariSourceMap.clear();
        } else if (scrutariSourceList.isEmpty()) {
            Map<String, ScrutariSourceStorage.Status> statusMap = ScrutariSourceStorage.fromString(sctServerStorage.getContent(SctServerStorage.STATUS_NAME));
            int length = scrutariSourceDefArray.length;
            for (int i = 0; i < length; i++) {
                ScrutariSourceDef scrutariSourceDef = scrutariSourceDefArray[i];
                String name = scrutariSourceDef.getName();
                InternalScrutariSource scrutariSource = new InternalScrutariSource(scrutariSourceDef);
                ScrutariSourceStorage.Status status = statusMap.get(name);
                if (status != null) {
                    scrutariSource.setState(status.getState());
                    scrutariSource.lastUpdate = status.getLastUpdate();
                }
                scrutariSourceList.add(scrutariSource);
                scrutariSourceMap.put(name, scrutariSource);
            }
        } else {
            Map<String, InternalScrutariSource> copyMap = new HashMap<String, InternalScrutariSource>();
            copyMap.putAll(scrutariSourceMap);
            scrutariSourceList.clear();
            scrutariSourceMap.clear();
            int length = scrutariSourceDefArray.length;
            for (int i = 0; i < length; i++) {
                ScrutariSourceDef scrutariSourceDef = scrutariSourceDefArray[i];
                String name = scrutariSourceDef.getName();
                InternalScrutariSource scrutariSource = copyMap.get(name);
                if (scrutariSource != null) {
                    scrutariSource.setScrutariSourceDef(scrutariSourceDef);
                } else {
                    scrutariSource = new InternalScrutariSource(scrutariSourceDef);
                }
                scrutariSourceList.add(scrutariSource);
                scrutariSourceMap.put(name, scrutariSource);
            }
        }
        store();
    }

    @Override
    public boolean collect(CollectErrorHandler collectErrorHandler, boolean forceDataDownload) {
        int length = scrutariSourceList.size();
        if (length == 0) {
            return false;
        }
        boolean dataChange = false;
        for (int i = 0; i < length; i++) {
            InternalScrutariSource scrutariSource = scrutariSourceList.get(i);
            CollectResult collectResult = CollectEngine.collectData(scrutariSource, sctServerStorage, collectErrorHandler, forceDataDownload);
            if (CollectUtils.hasChange(scrutariSource, collectResult)) {
                dataChange = true;
                scrutariSource.update(collectResult);
            }
        }
        store();
        if (forceDataDownload) {
            return true;
        } else {
            return dataChange;
        }
    }

    boolean setState(ScrutariSource scrutariSource, short state) {
        boolean done = ((InternalScrutariSource) scrutariSource).setState(state);
        if (done) {
            store();
        }
        return done;
    }

    private void store() {
        sctServerStorage.saveContent(SctServerStorage.STATUS_NAME, ScrutariSourceStorage.toString(this));
    }


    private class InternalScrutariSource implements ScrutariSource {

        private ScrutariSourceDef scrutariSourceDef;
        private short state = ScrutariSource.UNKNOWN_STATE;
        private int lastUpdate = DateInteger.NULL;

        private InternalScrutariSource(ScrutariSourceDef scrutariSourceDef) {
            this.scrutariSourceDef = scrutariSourceDef;
        }

        private InternalScrutariSource(ScrutariSource scrutariSource) {
            this.scrutariSourceDef = scrutariSource.getScrutariSourceDef();
            this.lastUpdate = scrutariSource.getLastUpdate();
            this.state = scrutariSource.getState();
        }

        @Override
        public ScrutariSourceDef getScrutariSourceDef() {
            return scrutariSourceDef;
        }

        @Override
        public short getState() {
            return state;
        }

        @Override
        public int getLastUpdate() {
            return lastUpdate;
        }

        private void setScrutariSourceDef(ScrutariSourceDef scrutariSourceDef) {
            if (!this.scrutariSourceDef.getUrl().equals(scrutariSourceDef.getUrl())) {
                state = ScrutariSource.UNKNOWN_STATE;
                lastUpdate = DateInteger.NULL;
            }
            this.scrutariSourceDef = scrutariSourceDef;
        }

        private boolean setState(short state) {
            if (this.state == state) {
                return false;
            }
            this.state = state;
            return true;
        }

        private void update(CollectResult collectResult) {
            this.state = collectResult.getState();
            this.lastUpdate = collectResult.getLastUpdateDateInteger();
        }

    }

}
