/* SctServer - Copyright (c) 2014-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.impl;

import fr.exemole.sctserver.api.ConfState;
import fr.exemole.sctserver.api.L10nManager;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.ScrutariSourceManager;
import fr.exemole.sctserver.api.SctServer;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceList;
import fr.exemole.sctserver.api.conf.ConfConstants;
import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.conf.ConfFileInfo;
import fr.exemole.sctserver.api.conf.ConfIOException;
import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.tools.log.UpdateLogHandler;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.ProcessMessage;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.TimeLog;
import net.scrutari.datauri.BaseURI;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.HistoryDataCoder;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.db.api.caches.DataCacheReaderFactory;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.db.tools.AttributeKeyCoderBuilder;
import net.scrutari.db.tools.DataCoderUtils;
import net.scrutari.db.tools.HistoryDataCoderBuilder;
import net.scrutari.db.tools.ScrutariDBBuilder;
import net.scrutari.xml.BaseXmlEngine;
import net.scrutari.xml.IndexationXmlEngine;
import net.scrutari.xml.xmlpull.XmlChecker;
import org.xmlpull.v1.XmlPullParserException;

/**
 *
 * @author Vincent Calame
 */
public final class SctServerImpl implements SctServer {

    private final String name;
    private final SctServerStorage sctServerStorage;
    private final L10nManagerImpl l10nManagerImpl;
    private final ScrutariSourceManagerImpl scrutariSourceManager;
    private Lang defaultLangUi;
    private ScrutariSessionImpl scrutariSession;
    private final ConfStateImpl confState = new ConfStateImpl();
    private boolean onReload = false;

    public SctServerImpl(String name, SctServerStorage sctServerStorage, Lang defaultLangUi) {
        if (defaultLangUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.name = name;
        this.sctServerStorage = sctServerStorage;
        this.defaultLangUi = defaultLangUi;
        this.l10nManagerImpl = new L10nManagerImpl();
        this.scrutariSourceManager = new ScrutariSourceManagerImpl(sctServerStorage);
        init();
    }

    private void init() {
        SctServerStorage.CacheCheck cacheCheck = sctServerStorage.checkLastCache(null);
        if (cacheCheck != null) {
            TimeLog timeLog = new TimeLog();
            ConfDirectory confDirectory = initConfDirectory();
            timeLog.log("confdirectory_init");
            scrutariSourceManager.reloadConf(confDirectory);
            timeLog.log("scrutarisourcemanager_loaded");
            AttributeKeyCoder attributeKeyCoder = AttributeKeyCoderBuilder.build(confDirectory.getAttributeDefListMap());
            timeLog.log("attributekeycoder_build");
            ScrutariDB newScrutariDB = ScrutariDBBuilder.build(cacheCheck.getScrutariDBName(), attributeKeyCoder, cacheCheck.getDataCacheReaderFactory(), cacheCheck.getLexieCacheReaderFactory(), timeLog);
            timeLog.log("scrutaridb_build");
            scrutariSession = ScrutariSessionImpl.newInstance(this, newScrutariDB, confDirectory, timeLog);
            timeLog.log("scrutarisession_build");
            timeLog.log("start_end");
            sctServerStorage.getLogStorage().log(SctLogConstants.START_LOG, SctLogConstants.TIME_LOG, timeLog.toString());
        } else {
            reload(INIT_TYPE);
        }
    }

    @Override
    public boolean reload(short type) {
        if (onReload) {
            return false;
        }
        onReload = true;
        ConfDirectory confDirectory = initConfDirectory();
        scrutariSourceManager.reloadConf(confDirectory);
        if (type != UPDATECONF_TYPE) {
            UpdateLogHandler updateLogHandler = new UpdateLogHandler();
            TimeLog timeLog = new TimeLog();
            boolean forceReload = (type == FORCE_TYPE);
            boolean done = scrutariSourceManager.collect(updateLogHandler, forceReload);
            timeLog.log("collect_done");
            if ((type != CRON_TYPE) || (done)) {
                reinitScrutariDB(confDirectory, updateLogHandler, timeLog);
                timeLog.log("update_end");
                sctServerStorage.getLogStorage().log(SctLogConstants.UPDATE_LOG, SctLogConstants.TIME_LOG, timeLog.toString());
                sctServerStorage.getLogStorage().log(SctLogConstants.UPDATE_LOG, SctLogConstants.DATA_LOG, updateLogHandler.toContentString());
            }
        } else {
            scrutariSession.reloadConf(confDirectory);
        }
        onReload = false;
        return true;
    }

    @Override
    public String getName() {
        return name;
    }



    @Override
    public synchronized ScrutariSession getScrutariSession() {
        return scrutariSession;
    }

    @Override
    public ScrutariSourceManager getScrutariInfoManager() {
        return scrutariSourceManager;
    }

    @Override
    public L10nManager getL10nManager() {
        return l10nManagerImpl;
    }

    @Override
    public SctServerStorage getSctServerStorage() {
        return sctServerStorage;
    }

    @Override
    public ConfState getConfState() {
        return confState;
    }

    @Override
    public Lang getDefaultLangUi() {
        return defaultLangUi;
    }

    public void setDefaultLangUi(Lang defaultLangUi) {
        if (defaultLangUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.defaultLangUi = defaultLangUi;
    }

    private void reinitScrutariDB(ConfDirectory confDirectory, UpdateLogHandler updateLogHandler, TimeLog timeLog) {
        ScrutariDBName scrutariDbName = ScrutariDBName.newName();
        HistoryDataCoder historyDataCoder = HistoryDataCoderBuilder.build(confDirectory.getInitialScrutariDataURICodeList(), sctServerStorage.getURITree(null));
        DataCoder encapsulatingDataCoder = DataCoderUtils.encapsulate(historyDataCoder);
        timeLog.log("historydatacoder_build");
        AttributeKeyCoder attributeKeyCoder = AttributeKeyCoderBuilder.build(confDirectory.getAttributeDefListMap());
        timeLog.log("attributekeycoder_build");
        DataCacheWriter dataCacheWriter = sctServerStorage.getDataCacheWriter(scrutariDbName);
        parseXml(dataCacheWriter, encapsulatingDataCoder, updateLogHandler, timeLog);
        DataCacheReaderFactory dataCacheReaderFactory = dataCacheWriter.endCacheWrite();
        timeLog.log("xml_cache");
        ScrutariDB newScrutariDB = ScrutariDBBuilder.build(scrutariDbName, attributeKeyCoder, dataCacheReaderFactory, sctServerStorage.getLexieCacheWriter(scrutariDbName), timeLog);
        timeLog.log("scrutaridb_build");
        sctServerStorage.saveURITree(historyDataCoder.toURITree(), null);
        if (historyDataCoder.hasNewURI()) {
            sctServerStorage.saveURITree(historyDataCoder.getNewURITree(), scrutariDbName);
        }
        timeLog.log("uritree_saved");
        Set<ScrutariDBName> remainingSet = new HashSet<ScrutariDBName>();
        remainingSet.add(newScrutariDB.getScrutariDBName());
        ScrutariSessionImpl newScrutariSession = ScrutariSessionImpl.newInstance(this, newScrutariDB, confDirectory, timeLog);
        timeLog.log("scrutarisession_build");
        ScrutariSessionImpl oldScrutariSession = scrutariSession;
        if (oldScrutariSession != null) {
            remainingSet.add(oldScrutariSession.getScrutariDB().getScrutariDBName());
        }
        synchronized (this) {
            scrutariSession = newScrutariSession;
        }
        sctServerStorage.cleanCache(remainingSet);

    }

    private void parseXml(DataCacheWriter dataCacheWriter, DataCoder dataCoder, UpdateLogHandler updateProcessLog, TimeLog timeLog) {
        XmlChecker xmlChecker = new XmlChecker();
        BaseXmlEngine baseXmlEngine = new BaseXmlEngine(dataCacheWriter, dataCoder, updateProcessLog);
        ScrutariSourceList scrutariSourceList = scrutariSourceManager.getScrutariSourceList();
        int count = scrutariSourceList.getScrutariSourceCount();
        Map<Integer, BaseURI> validBaseURIMap = new HashMap<Integer, BaseURI>();
        for (int i = 0; i < count; i++) {
            ScrutariSource scrutariSource = scrutariSourceList.getScrutariSource(i);
            String scrutariSourceName = scrutariSource.getScrutariSourceDef().getName();
            updateProcessLog.setCurrentScrutariSource(scrutariSourceName);
            timeLog.addSection(scrutariSourceName);
            InputStream inputStream = sctServerStorage.getScrutariData(scrutariSourceName);
            if (inputStream != null) {
                boolean done = false;
                try {
                    done = xmlChecker.check(inputStream);
                } catch (XmlPullParserException xppe) {
                    scrutariSourceManager.setState(scrutariSource, ScrutariSource.DATA_NOTXML_STATE);
                    updateProcessLog.xmlPullParserException(xppe);
                } catch (IOException ioe) {
                    scrutariSourceManager.setState(scrutariSource, ScrutariSource.DATA_NOTXML_STATE);
                    updateProcessLog.xmlPullParserException(ioe);
                } finally {
                    try {
                        inputStream.close();
                    } catch (IOException ioe) {

                    }
                }
                if (done) {
                    inputStream = sctServerStorage.getScrutariData(scrutariSourceName);
                    try {
                        BaseURI baseURI = baseXmlEngine.run(scrutariSourceName, inputStream);
                        validBaseURIMap.put(i, baseURI);
                    } catch (XmlPullParserException xppe) {
                    } catch (IOException ioe) {
                    } finally {
                        try {
                            inputStream.close();
                        } catch (IOException ioe) {

                        }
                    }
                }
            }
            timeLog.log("basedata_parsed");
        }
        IndexationXmlEngine indexationXmlEngine = new IndexationXmlEngine(dataCacheWriter, dataCoder, updateProcessLog);
        for (int i = 0; i < count; i++) {
            BaseURI baseURI = validBaseURIMap.get(i);
            if (baseURI != null) {
                ScrutariSource scrutariSource = scrutariSourceList.getScrutariSource(i);
                String scrutariSourceName = scrutariSource.getScrutariSourceDef().getName();
                updateProcessLog.setCurrentScrutariSource(scrutariSourceName);
                timeLog.addSection(scrutariSourceName);
                InputStream inputStream = sctServerStorage.getScrutariData(scrutariSourceName);
                if (inputStream != null) {
                    try {
                        indexationXmlEngine.run(inputStream, baseURI);
                    } catch (XmlPullParserException xppe) {
                    } catch (IOException ioe) {
                    } finally {
                        try {
                            inputStream.close();
                        } catch (IOException ioe) {

                        }
                    }
                }
            }
            timeLog.log("indexationdata_parsed");
        }
    }

    private ConfDirectory initConfDirectory() {
        ConfDirectory confDirectory = sctServerStorage.getConfDirectory();
        try {
            confDirectory.reload();
            checkConfLog(confDirectory, ConfConstants.SOURCES_NAME);
            checkConfLog(confDirectory, ConfConstants.ATTRIBUTES_NAME);
            checkConfLog(confDirectory, ConfConstants.CATEGORIES_NAME);
            checkConfLog(confDirectory, ConfConstants.URI_CODES_NAME);
            checkConfLog(confDirectory, ConfConstants.FEED_NAME);
            checkConfLog(confDirectory, ConfConstants.OPTIONS_NAME);
            confState.setOkState();
        } catch (ConfIOException cioe) {
            confState.setGlobalConfigException(cioe);
        }
        return confDirectory;
    }

    private void checkConfLog(ConfDirectory confDirectory, String name) {
        ConfFileInfo confFileInfo = confDirectory.getConfFileInfo(name);
        if (!confFileInfo.isHere()) {
            confState.setFileState(name, ConfConstants.FILE_MISSING);
        } else {
            ProcessMessageList processMessageList = confFileInfo.getProcessMessageList();
            int count = processMessageList.getProcessMessageCount();
            if (count > 0) {
                int errorType = 0;
                for (int i = 0; i < count; i++) {
                    ProcessMessage processMessage = processMessageList.getProcessMessage(i);
                    String category = processMessage.getCategory();
                    if (category.equals(SctLogConstants.MALFORMED_XML_CATEGORY)) {
                        errorType = 2;
                        break;
                    } else if (category.equals(SctLogConstants.SEVERE_CATEGORY)) {
                        errorType = 1;
                    }
                }
                if (errorType == 2) {
                    confState.setFileState(name, ConfConstants.FILE_XML_MALFORMED);
                } else if (errorType == 1) {
                    confState.setFileState(name, ConfConstants.FILE_XML_ERROR);
                } else {
                    confState.setFileState(name, ConfConstants.FILE_XML_WARNING);
                }
            } else {
                confState.setFileState(name, ConfConstants.FILE_OK);
            }
            StringBuilder buf = new StringBuilder();
            try {
                LogUtils.append(buf, confFileInfo.getProcessURI(), processMessageList);
            } catch (IOException ioe) {
            }
            sctServerStorage.getLogStorage().log(SctLogConstants.CONF_LOG, name, buf.toString());
        }

    }

}
