/* SctServer - Copyright (c) 2009-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.collect.ScrutariSourceDefBuilder;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.LinkedHashMap;
import java.util.Map;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SourcesDOMReader {

    private final ProcessMessageHandler errorHandler;
    private final Map<String, ScrutariSourceDef> scrutariSourceDefMap = new LinkedHashMap<String, ScrutariSourceDef>();

    public SourcesDOMReader(ProcessMessageHandler errorHandler) {
        this.errorHandler = errorHandler;
    }

    public ScrutariSourceDef[] readSources(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("source")) {
                    addSource(element);
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
        return scrutariSourceDefMap.values().toArray(new ScrutariSourceDef[scrutariSourceDefMap.size()]);
    }

    private void addSource(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            errorHandler.addMessage(SctLogConstants.SEVERE_CATEGORY, "missingNameAttribute", name);
            return;
        }
        if (!StringUtils.isTechnicalName(name, true)) {
            errorHandler.addMessage(SctLogConstants.SEVERE_CATEGORY, "wrongNameAttribute", name);
            return;
        }
        if (scrutariSourceDefMap.containsKey(name)) {
            errorHandler.addMessage(SctLogConstants.WARNING_CATEGORY, "doublonName", name);
            return;
        }
        ScrutariSourceDefBuilder builder = new ScrutariSourceDefBuilder(name);
        boolean done = false;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("info-url")) {
                    String urlString = XMLUtils.getData(element);
                    if (urlString.length() == 0) {
                        ErrorUtils.emptyElementError(errorHandler, "info-url");
                    } else {
                        try {
                            URL url = new URL(urlString);
                            builder.setInfoUrl(url);
                            done = true;
                        } catch (MalformedURLException mue) {
                            errorHandler.addMessage(SctLogConstants.SEVERE_CATEGORY, "malformedUrl", urlString + " (" + mue.getMessage() + ")");
                        }
                    }
                } else if (tagName.equals("scrutaridata-url")) {
                    String urlString = XMLUtils.getData(element);
                    if (urlString.length() == 0) {
                        ErrorUtils.emptyElementError(errorHandler, "scrutaridata-url");
                    } else {
                        int frequency = 1;
                        String freqString = element.getAttribute("freq");
                        if (freqString.length() > 0) {
                            try {
                                frequency = Integer.parseInt(freqString);
                            } catch (NumberFormatException nfe) {
                                errorHandler.addMessage(SctLogConstants.WARNING_CATEGORY, "wrongFrequencyAttribute", freqString);
                            }
                        }
                        try {
                            URL url = new URL(urlString);
                            builder.setScrutariDataUrl(url, frequency);
                            done = true;
                        } catch (MalformedURLException mue) {
                            errorHandler.addMessage(SctLogConstants.SEVERE_CATEGORY, "malformedUrl", urlString + " (" + mue.getMessage() + ")");
                        }
                    }
                }
            }
        }
        if (!done) {
            errorHandler.addMessage(SctLogConstants.SEVERE_CATEGORY, "missingUrlElement", name);
        } else {
            scrutariSourceDefMap.put(name, builder.toScrutariSourceDef());
        }
    }

}
