/* SctServer - Copyright (c) 2009-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.conf.ConfFileInfo;
import fr.exemole.sctserver.api.conf.ConfIOException;
import fr.exemole.sctserver.api.feed.SctFeedMetadata;
import java.io.File;
import java.io.IOException;
import java.util.Map;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.logging.ProcessMessageList;
import net.scrutari.db.api.codes.ScrutariDataURICodeList;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.api.options.PertinencePonderation;


/**
 *
 * @author Vincent Calame
 */
public class ConfDirectoryFactory {

    private ConfDirectoryFactory() {
    }

    public static ConfDirectory fromDirectory(File confDir) {
        return new InternalConfDirectory(confDir);
    }


    private static class InternalConfDirectory implements ConfDirectory {

        private final File confDir;
        private final InternalConfFileInfo sourcesFileInfo = new InternalConfFileInfo();
        private final InternalConfFileInfo categoriesFileInfo = new InternalConfFileInfo();
        private final InternalConfFileInfo searchLogFilterFileInfo = new InternalConfFileInfo();
        private final InternalConfFileInfo uriCodesFileInfo = new InternalConfFileInfo();
        private final InternalConfFileInfo feedFileInfo = new InternalConfFileInfo();
        private final InternalConfFileInfo attributesFileInfo = new InternalConfFileInfo();
        private final InternalConfFileInfo optionsFileInfo = new InternalConfFileInfo();
        private ScrutariSourceDef[] scrutariSourceDefArray = null;
        private CategoryDef[] categoryDefArray = null;
        private PertinencePonderation globalPertinencePonderation;
        private ScrutariDataURICodeList initialScrutariDataURICodeList;
        private SctFeedMetadata sctFeedMetadata;
        private Map<String, AttributeDefList> attributeDefListMap;

        private InternalConfDirectory(File confDir) {
            this.confDir = confDir;
        }

        @Override
        public ScrutariSourceDef[] getScrutariSourceDefArray() {
            return scrutariSourceDefArray;
        }

        @Override
        public CategoryDef[] getCategoryDefArray() {
            return categoryDefArray;
        }

        @Override
        public PertinencePonderation getGlobalPertinencePonderation() {
            return globalPertinencePonderation;
        }

        @Override
        public ScrutariDataURICodeList getInitialScrutariDataURICodeList() {
            return initialScrutariDataURICodeList;
        }

        @Override
        public SctFeedMetadata getSctFeedMetadata() {
            return sctFeedMetadata;
        }

        @Override
        public Map<String, AttributeDefList> getAttributeDefListMap() {
            return attributeDefListMap;
        }

        @Override
        public void reload() throws ConfIOException {
            clear();
            if (!confDir.exists()) {
                return;
            }
            try {
                if (!confDir.isDirectory()) {
                    throw new ConfIOException("confDirectory is not a directory : " + confDir.getPath());
                }
                for (File f : confDir.listFiles()) {
                    if (!f.isDirectory()) {
                        String name = f.getName();
                        if ((name.length() > 4) && (name.endsWith(".xml"))) {
                            String baseName = name.substring(0, name.length() - 4);
                            if (baseName.equals(SOURCES_NAME)) {
                                initSources(f);
                            } else if (baseName.equals(CATEGORIES_NAME)) {
                                initCategories(f);
                            } else if (baseName.equals(URI_CODES_NAME)) {
                                initUriCodes(f);
                            } else if (baseName.equals(FEED_NAME)) {
                                initFeed(f);
                            } else if (baseName.equals(ATTRIBUTES_NAME)) {
                                initAttributes(f);
                            } else if (baseName.equals(OPTIONS_NAME)) {
                                initOptions(f);
                            }
                        }
                    }
                }
            } catch (SecurityException se) {
                throw new ConfIOException(se);
            } catch (IOException ioe) {
                throw new ConfIOException(ioe);
            } catch (ParserConfigurationException pe) {
                throw new ConfIOException(pe);
            }
        }

        private void clear() {
            sourcesFileInfo.clear();
            categoriesFileInfo.clear();
            searchLogFilterFileInfo.clear();
            uriCodesFileInfo.clear();
            feedFileInfo.clear();
            scrutariSourceDefArray = null;
            categoryDefArray = null;
            initialScrutariDataURICodeList = null;
            sctFeedMetadata = null;
            attributeDefListMap = null;
        }

        private void initSources(File f) throws IOException, ParserConfigurationException {
            sourcesFileInfo.setHere(true);
            SourcesConf sourcesConf = SourcesConf.load(f);
            sourcesFileInfo.setProcessMessageList(sourcesConf.getProcessMessageList());
            sourcesFileInfo.setProcessURI(sourcesConf.getProcessURI());
            scrutariSourceDefArray = sourcesConf.getScrutariSourceDefArray();
        }

        private void initCategories(File f) throws IOException, ParserConfigurationException {
            categoriesFileInfo.setHere(true);
            CategoriesConf categoriesConf = CategoriesConf.load(f);
            categoriesFileInfo.setProcessMessageList(categoriesConf.getProcessMessageList());
            categoriesFileInfo.setProcessURI(categoriesConf.getProcessURI());
            categoryDefArray = categoriesConf.getCategoryDefArray();
        }

        private void initUriCodes(File f) throws IOException, ParserConfigurationException {
            uriCodesFileInfo.setHere(true);
            UriCodesConf uriCodesConf = UriCodesConf.load(f);
            uriCodesFileInfo.setProcessMessageList(uriCodesConf.getProcessMessageList());
            uriCodesFileInfo.setProcessURI(uriCodesConf.getProcessURI());
            initialScrutariDataURICodeList = uriCodesConf.getScrutariDataURICodeList();
        }

        private void initFeed(File f) throws IOException, ParserConfigurationException {
            feedFileInfo.setHere(true);
            FeedConf feedConf = FeedConf.load(f);
            feedFileInfo.setProcessMessageList(feedConf.getProcessMessageList());
            feedFileInfo.setProcessURI(feedConf.getProcessURI());
            sctFeedMetadata = feedConf.getSctFeedMetadata();
        }

        private void initAttributes(File f) throws IOException, ParserConfigurationException {
            attributesFileInfo.setHere(true);
            AttributesConf attributesConf = AttributesConf.load(f);
            attributesFileInfo.setProcessMessageList(attributesConf.getProcessMessageList());
            attributesFileInfo.setProcessURI(attributesConf.getProcessURI());
            attributeDefListMap = attributesConf.getAttributeDefListMap();
        }

        private void initOptions(File f) throws IOException, ParserConfigurationException {
            optionsFileInfo.setHere(true);
            OptionsConf optionsConf = OptionsConf.load(f);
            optionsFileInfo.setProcessMessageList(optionsConf.getProcessMessageList());
            optionsFileInfo.setProcessURI(optionsConf.getProcessURI());
            globalPertinencePonderation = optionsConf.getGlobalPertinencePonderation();
        }

        @Override
        public ConfFileInfo getConfFileInfo(String name) {
            if (name.equals(SOURCES_NAME)) {
                return sourcesFileInfo;
            } else if (name.equals(CATEGORIES_NAME)) {
                return categoriesFileInfo;
            } else if (name.equals(URI_CODES_NAME)) {
                return uriCodesFileInfo;
            } else if (name.equals(FEED_NAME)) {
                return feedFileInfo;
            } else if (name.equals(ATTRIBUTES_NAME)) {
                return attributesFileInfo;
            } else if (name.equals(OPTIONS_NAME)) {
                return optionsFileInfo;
            } else {
                throw new IllegalArgumentException("Unknown name : " + name);
            }
        }

    }


    private static class InternalConfFileInfo implements ConfFileInfo {

        private boolean here = false;
        private String processURI;
        private ProcessMessageList processMessageList;

        private InternalConfFileInfo() {
        }

        private void clear() {
            this.here = false;
            this.processURI = null;
            this.processMessageList = null;
        }

        @Override
        public boolean isHere() {
            return here;
        }

        @Override
        public String getProcessURI() {
            return processURI;
        }

        @Override
        public ProcessMessageList getProcessMessageList() {
            return processMessageList;
        }

        private void setHere(boolean here) {
            this.here = here;
        }

        private void setProcessMessageList(ProcessMessageList processMessageList) {
            this.processMessageList = processMessageList;
        }

        private void setProcessURI(String processURI) {
            this.processURI = processURI;
        }

    }

}
