/* UtilLib - Copyright (c) 2009-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.awt.Color;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.text.LibelleUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public final class AttributeUtils implements AttributeConstants {

    public final static AttributeDefList EMPTY_ATTRIBUTEDEFLIST = new EmptyAttributeDefList();
    public final static ChangeAttributeHolder EMPTY_CHANGEATTRIBUTEHOLDER = new EmptyChangeAttributeHolder();
    public final static AttributeHolder EMPTY_ATTRIBUTEHOLDER = EMPTY_CHANGEATTRIBUTEHOLDER;

    private AttributeUtils() {
    }

    public static boolean isEqual(Attribute attr1, Attribute attr2) {
        boolean test1 = attr1.getAttributeKey().equals(attr2.getAttributeKey());
        if (!test1) {
            return false;
        }
        int valueCount = attr1.getValueCount();
        if (valueCount != attr2.getValueCount()) {
            return false;
        }
        for (int i = 0; i < valueCount; i++) {
            if (!attr1.getValue(i).equals(attr2.getValue(i))) {
                return false;
            }
        }
        return true;
    }

    public static AttributeHolder toSingletonAttributeHolder(Attribute attribute) {
        if (attribute == null) {
            throw new IllegalArgumentException("attribute is null");
        }
        return new SingletonAttributeHolder(attribute);
    }

    public static AttributeDef toDefaultAttributeDef(AttributeKey attributeKey) {
        if (attributeKey == null) {
            throw new IllegalArgumentException("attributeKey is null");
        }
        return new DefaultAttributeDef(attributeKey);
    }

    public static AttributeDefList toAttributeDefList(List<AttributeDef> list) {
        AttributeDef[] array = list.toArray(new AttributeDef[list.size()]);
        return new ArrayAttributeDefList(array);
    }

    public static void addAttributes(XMLWriter xmlWriter, AttributeHolder attributeHolder) throws IOException {
        int count = attributeHolder.getAttributeCount();
        for (int i = 0; i < count; i++) {
            addAttribute(xmlWriter, attributeHolder.getAttributeByIndex(i));
        }
    }

    public static List<CleanedString> readValues(Element element) {
        AttributeElementHandler attributeElementHandler = new AttributeElementHandler();
        DOMUtils.readChildren(element, attributeElementHandler);
        return attributeElementHandler.list;
    }

    private static void addAttribute(XMLWriter xmlWriter, Attribute attribute) throws IOException {
        AttributeKey key = attribute.getAttributeKey();
        String nameSpace = key.getNameSpace();
        if (nameSpace.equals(AttributeKey.TRANSIENT_NAMESPACE)) {
            return;
        }
        xmlWriter.startOpenTag("attr");
        xmlWriter.addAttribute("ns", nameSpace);
        xmlWriter.addAttribute("key", key.getLocalKey());
        xmlWriter.endOpenTag();
        int valueCount = attribute.getValueCount();
        for (int i = 0; i < valueCount; i++) {
            xmlWriter.addSimpleElement("val", attribute.getValue(i));
        }
        xmlWriter.closeTag("attr");
    }


    private static class EmptyChangeAttributeHolder implements ChangeAttributeHolder {

        private EmptyChangeAttributeHolder() {
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            return null;
        }

        @Override
        public int getAttributeCount() {
            return 0;
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public int getRemovedAttributeCount() {
            return 0;
        }

        @Override
        public AttributeKey getRemovedAttributeKey(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class SingletonAttributeHolder implements AttributeHolder {

        private final Attribute attribute;

        private SingletonAttributeHolder(Attribute attribute) {
            this.attribute = attribute;
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            if (attribute.getAttributeKey().equals(key)) {
                return attribute;
            } else {
                return null;
            }
        }

        @Override
        public int getAttributeCount() {
            return 1;
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return attribute;
        }

    }


    private static class DefaultAttributeDef implements AttributeDef {

        private final AttributeKey attributeKey;
        private final LibelleHolder libelleHolder;

        private DefaultAttributeDef(AttributeKey attributeKey) {
            this.attributeKey = attributeKey;
            this.libelleHolder = LibelleUtils.EMPTY_CHANGELIBELLEHOLDER;
        }

        @Override
        public AttributeKey getAttributeKey() {
            return attributeKey;
        }

        @Override
        public int getScopeCount() {
            return 0;
        }

        @Override
        public String getScope(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public String getFormatType() {
            return "";
        }

        @Override
        public Object getFormatObject() {
            return null;
        }

        @Override
        public LibelleHolder getLibelleHolder() {
            return libelleHolder;
        }

    }


    private static class EmptyAttributeDefList implements AttributeDefList {

        private EmptyAttributeDefList() {
        }

        @Override
        public int getAttributeDefCount() {
            return 0;
        }

        @Override
        public AttributeDef getAttributeDef(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArrayAttributeDefList implements AttributeDefList {

        private final AttributeDef[] array;

        private ArrayAttributeDefList(AttributeDef[] array) {
            this.array = array;
        }

        @Override
        public int getAttributeDefCount() {
            return array.length;
        }

        @Override
        public AttributeDef getAttributeDef(int i) {
            return array[i];
        }

    }

    public static boolean testAttribute(Attribute attribute, String formatType) {
        int count = attribute.getValueCount();
        for (int i = 0; i < count; i++) {
            Object obj = formatValue(attribute.getValue(i), formatType);
            if (obj == null) {
                return false;
            }
        }
        return true;
    }

    public static Object formatValue(String value, String formatType) {
        if (formatType.equals(COLOR_FORMAT)) {
            try {
                return Color.decode(value);
            } catch (NumberFormatException nfe) {
                return null;
            }
        } else if (formatType.equals(URL_FORMAT)) {
            try {
                return new URL(value);
            } catch (MalformedURLException mue) {
                return null;
            }
        } else if (formatType.startsWith(INTEGER_PREFIX)) {
            try {
                Integer itg = Integer.parseInt(value);
                if (formatType.equals(POSITIVE_INTEGER_FORMAT)) {
                    if (itg < 0) {
                        return null;
                    }
                } else if (formatType.equals(STRICTPOSITIVE_INTEGER_FORMAT)) {
                    if (itg <= 0) {
                        return null;
                    }
                }
                return itg;
            } catch (NumberFormatException nfe) {
                return null;
            }
        } else if (formatType.equals(DATATION_FORMAT)) {
            try {
                return DateInteger.parse(value);
            } catch (DateIntegerException die) {
                return null;
            }
        }
        return value;
    }


    private static class AttributeElementHandler implements ElementHandler {

        private final List<CleanedString> list = new ArrayList<CleanedString>();

        private AttributeElementHandler() {
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("val")) {
                CleanedString cs = XMLUtils.toCleanedString(element);
                if (cs != null) {
                    list.add(cs);
                }
            }
        }

    }

}
