/* UtilLib - Copyright (c) 2011-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.io.IOException;
import java.text.ParseException;
import java.util.List;
import net.mapeadores.util.instruction.Argument;
import net.mapeadores.util.instruction.Instruction;
import net.mapeadores.util.instruction.InstructionErrorHandler;
import net.mapeadores.util.instruction.InstructionParser;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public final class LibelleUtils {

    public final static ChangeLibelleHolder EMPTY_CHANGELIBELLEHOLDER = new EmptyChangeLibelleHolder();

    private LibelleUtils() {
    }

    public static boolean areEquals(Libelle lib1, Libelle lib2) {
        if (lib1 == null) {
            return (lib2 == null);
        } else if (lib2 == null) {
            return false;
        }
        if (!lib1.getLang().equals(lib2.getLang())) {
            return false;
        }
        return lib1.getLibelleString().equals(lib2.getLibelleString());
    }

    /*
     * Retourne nul si vide.
     * si l'instruction est incorrecte, retourne un libelleHolder vide avec un message d'erreur par défaut.
     * l'instruction doit être sous la forme fr="",en="", etc.
     */
    public static LibelleHolder parseInstruction(String s) throws ParseException {
        if (s == null) {
            return null;
        }
        s = s.trim();
        if (s.length() == 0) {
            return null;
        }
        if (s.indexOf('=') == -1) {
            throw new ParseException("#missingEqualCharacter", 0);
        }
        InstructionParseErrorHandler errorHandler = new InstructionParseErrorHandler();
        Instruction instruction = InstructionParser.parse(s, errorHandler);
        if (errorHandler.hasError()) {
            throw new ParseException(errorHandler.getErrorMessage(), 0);
        }
        if (instruction == null) {
            return null;
        }
        ChangeLibelleHolderBuilder builder = new ChangeLibelleHolderBuilder();
        int length = instruction.getArgumentCount();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            try {
                Lang lang = Lang.parse(argument.getKey());
                builder.putLibelle(lang, argument.getValue());
            } catch (ParseException pe) {
            }
        }
        return builder.toChangeLibelleHolder();
    }

    public static LibelleHolder toLibelleHolder(List<Libelle> libelleList) {
        InternalLibelleHolder libelleHolder = new InternalLibelleHolder();
        for (Libelle libelle : libelleList) {
            libelleHolder.innerPutLibelle(libelle);
        }
        return libelleHolder;
    }

    /**
     * Retourne nul si vide
     */
    public static Libelle readLib(Element element) throws ParseException {
        Lang lang = Lang.parse(element.getAttribute("xml:lang"));
        CleanedString libString = DOMUtils.contentToCleanedString(element);
        if (libString == null) {
            return null;
        }
        return toLibelle(lang, libString);
    }

    public static Libelle toLibelle(Lang lang, CleanedString cleanedString) {
        if (lang == null) {
            throw new IllegalArgumentException("lang is null");
        }
        if (cleanedString == null) {
            throw new IllegalArgumentException("cleanedString is null");
        }
        return new InternalLibelle(lang, cleanedString.toString());
    }

    public static String checkLibelleString(LibelleHolder libelleHolder, Lang lang, String defaultString) {
        Libelle libelle = getLangPartCheckedLibelle(libelleHolder, lang);
        if (libelle != null) {
            return libelle.getLibelleString();
        } else {
            return defaultString;
        }
    }

    public static Libelle getLangPartCheckedLibelle(LibelleHolder libelleHolder, Lang lang) {
        Libelle libelle = libelleHolder.getLibelleByLang(lang);
        if ((libelle == null) && (!lang.isRootLang())) {
            libelle = libelleHolder.getLibelleByLang(lang.getRootLang());
        }
        return libelle;
    }

    public static Libelle getFirstLibelle(LibelleHolder libelleHolder) {
        int count = libelleHolder.getLibelleCount();
        if (count > 0) {
            return libelleHolder.getLibelleByIndex(0);
        } else {
            return null;
        }
    }

    public static Libelle getFirstLibelle(LibelleHolder libelleHolder, Lang[] langArray) {
        for (Lang lang : langArray) {
            Libelle libelle = libelleHolder.getLibelleByLang(lang);
            if (libelle != null) {
                return libelle;
            }
        }
        return null;
    }

    public static Libelle getFirstLibelle(LibelleHolder libelleHolder, LangList langList) {
        int length = langList.getLangCount();
        for (int i = 0; i < length; i++) {
            Lang lang = langList.getLang(i);
            Libelle libelle = libelleHolder.getLibelleByLang(lang);
            if (libelle != null) {
                return libelle;
            }
        }
        return null;
    }

    public static void addLibelles(XMLWriter xmlWriter, LibelleHolder libelleHolder) throws IOException {
        addLibelles(xmlWriter, libelleHolder, null);
    }

    public static void addLibelles(XMLWriter xmlWriter, LibelleHolder libelleHolder, LangFilter langFilter) throws IOException {
        int libelleCount = libelleHolder.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle lib = libelleHolder.getLibelleByIndex(i);
            if ((langFilter == null) || (langFilter.accept(lib.getLang()))) {
                addLibelle(xmlWriter, lib.getLang(), lib.getLibelleString());
            }
        }
    }

    private static void addLibelle(XMLWriter xmlWriter, Lang lang, String lib) throws IOException {
        xmlWriter.startOpenTag("lib");
        XMLUtils.addXmlLangAttribute(xmlWriter, lang);
        xmlWriter.endOpenTag();
        xmlWriter.addText(lib);
        xmlWriter.closeTag("lib", false);
    }


    private static class EmptyChangeLibelleHolder implements ChangeLibelleHolder {

        private EmptyChangeLibelleHolder() {
        }

        @Override
        public int getLibelleCount() {
            return 0;
        }

        @Override
        public Libelle getLibelleByIndex(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public Libelle getLibelleByLang(Lang lang) {
            return null;
        }

        @Override
        public int getRemovedLangCount() {
            return 0;
        }

        @Override
        public Lang getRemovedLang(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    public static class InstructionParseErrorHandler implements InstructionErrorHandler {

        private String errorMessage;

        private InstructionParseErrorHandler() {

        }

        @Override
        public void invalidAsciiCharacterError(String part, int row, int col) {
            errorMessage = "#invalidAsciiCharacter: " + part + " (" + row + "," + col + ")#";
        }

        @Override
        public void invalidEndCharacterError(String part, int row, int col) {
            errorMessage = "#invalidEndCharacter: " + part + " (" + row + "," + col + ")#";
        }

        @Override
        public void invalidSeparatorCharacterError(String part, int row, int col) {
            errorMessage = "#invalidSeparatorCharacter: " + part + " (" + row + "," + col + ")#";
        }

        private boolean hasError() {
            return (errorMessage != null);
        }

        private String getErrorMessage() {
            return errorMessage;
        }

    }


    private static class InternalLibelleHolder extends DefaultLibelleHolder {

        public InternalLibelleHolder() {
            super();
        }

    }


    private static class InternalLibelle implements Libelle {

        private final Lang lang;
        private final String libelleString;

        private InternalLibelle(Lang lang, String libelleString) {
            this.lang = lang;
            this.libelleString = libelleString;
        }

        @Override
        public Lang getLang() {
            return lang;
        }

        @Override
        public String getLibelleString() {
            return libelleString;
        }

    }

}
